using System;
using System.Xml;
using System.Collections;
using System.IO;
using System.Reflection;
using Microsoft.Win32; 
using Vha.Vss.Utility;
using System.Configuration;

namespace Vha.Vss.Utility
{
	/// <summary>
	/// Diffrent Component Name Recognition.
	/// </summary>
	public enum ComponentNameId
	{
		/// <summary>Time Keeping Module of VSS</summary>
		TimeKeeping,
		/// <summary>Services Directory Module of VSS</summary>
		ServicesDirectory,
		/// <summary>National Officials Module of VSS</summary>
		NationalOfficials,
		/// <summary>Games Module of VSS</summary>
		Games
	}
  ///<summary>
  ///The AppConfiguration class represents a single application's configuration file.  
  ///All configuration information for all VSS applications are accessible through this object.
  ///</summary>
  public class AppConfiguration : IConfigurationSectionHandler
  {
    // internal properties

    private String      m_ConfigFilename;
    private XmlDocument m_xmlDoc;

    private VhaLog myLog;  // Logging object used by Utility.dll for reporting errors

    ///<summary>
    ///Constructs a AppConfiguration object and initializes the internal XmlDocument.
    ///</summary>
    ///<Params>
    ///None.
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///This constructor is used by ConfigurationSectionHandler objects and
    ///installer classes.  Applications should just call the static member
    ///GetAppConfiguration to the get an instantiated object.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Microsoft.eSupport.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration();
    ///myConfigFile.Load("SomeConfigFile.config");
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///<para>For use within VHA components see the following ...
    ///</para>
    /// <seealso cref="GetAppConfiguration()"/>
    ///</References>
    public AppConfiguration() 
    {
      m_xmlDoc = new XmlDocument();
    } // end of Constructor AppConfiguration()


    ///<summary>
    ///Constructor that sets the ConfigFilename property.  
    ///</summary>
    ///<Params>
    ///<param type="String" name="ConfigFilename" Required="Mandatory">
    ///Application.config filename to be loaded and used by the AppConfiguration object.
    ///</param>
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///Useful by applications for one step initializatin of the object.  Still requires a 
    ///call to Load() to perform the actual read of the file.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration(@"d:\Vha.Vss\UI\web.config");
    ///myConfigFile.Load();
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="AppConfiguration(String)"/>
    ///</References>
    public AppConfiguration(String ConfigFilename)
    {
      m_xmlDoc = new XmlDocument();
      m_ConfigFilename = ConfigFilename;
    } // end of Constructor AppConfiguration(String)

    /// <summary>
    /// Public static method to be called to retrieve an AppConfiguration object that
    /// has been loaded with the application specific .config file data.
    /// </summary>
    ///<Params>
    ///None.
    ///</Params>
    ///<returns type="AppConfiguration">
    ///Returns an object that contains the application configuration data.
    ///</returns>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = AppConfiguration.GetAppConfiguration();
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="AppConfiguration(String)"/>
    ///</References>
    public static AppConfiguration GetAppConfiguration()
    {
      Object obj = ConfigurationSettings.GetConfig("VssConfiguration");
      if (obj == null)
      {
        throw new VhaException("The call to the URT's  ConfigurationSettings.GetConfig() returned a null object.");
      }
      return (AppConfiguration)obj;
    }

    /// <summary>
    /// Public static method to be called to retrieve an AppConfiguration object that
    /// has been loaded with explicitly specified configuration settings.
    /// </summary>
    ///<Params>
    ///<param type="String" name="configSettingsNodeName" Required="Mandatory">
    ///Name of the XML node that contains the configuration setting to be loaded.  
    ///Vss components should all use the &lt;VssConfiguration&gt; name and therefore
    ///do not use this API for intialization.
    ///</param>
    ///</Params>
    ///<returns type="AppConfiguration">
    ///Returns an object that contains the application configuration data.
    ///</returns>
    ///<remarks>
    ///Use of this method assumes a properly formed .config file. See example below.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///[config file requires this entry]
    ///&lt;configSections&gt;
    ///  &lt;section name="CustomConfiguration" type="Vha.Vss.Utility.AppConfiguration,Vss.Utility" /&gt;
    ///&lt;/configSections&gt;
    ///...
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = AppConfiguration.GetAppConfiguration("CustomConfig");
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="AppConfiguration(String)"/>
    ///</References>
    public static AppConfiguration GetAppConfiguration(String configSettingsNodeName)
    {
      Object obj = ConfigurationSettings.GetConfig(configSettingsNodeName);
      if (obj == null)
      {
        throw new VhaException("The call to the URT's  ConfigurationSettings.GetConfig() returned a null object.");
      }
      return (AppConfiguration)obj;
    }


  
    ///<summary>
    ///The filename that will be read/written per instantiation of the component.
    ///</summary>
    ///<Params>
    ///None.
    ///</Params>
    ///<returns type="String">
    ///Configuration filename.
    ///</returns>
    ///<remarks>
    ///Read/Write property.  The application or component must specify this filename either through this property
    ///or via the constructor in order to load the file.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration();
    ///myConfigFile.ConfigFilename = @"d:\Vha.Vss\UI\web.config";
    ///myConfigFile.Load();
    ///Console.WriteLine( "\nHere is the configuration file specific information..." 
    ///  + "\n\tComponentName:\t\t {0}"
    ///  + "\n\tComponentInstanceName:\t {1}"
    ///  + "\n\tComponentVersion:\t {2}"
    ///  + "\n\tInstalledToPath:\t {3}"
    ///  + "\n\tInstalledFromPath:\t {4}"
    ///  + "\n\tConfigFilename:\n\t{5}",
    ///  myConfigFile["ComponentName"],
    ///  myConfigFile["ComponentInstanceName"],
    ///  myConfigFile["ComponentVersion"],
    ///  myConfigFile["InstalledToPath"],
    ///  myConfigFile["InstalledFromPath"],
    ///  myConfigFile["ConfigFilename"]);
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public String ConfigFilename
    {
      get { return m_ConfigFilename; }
      set { m_ConfigFilename = value; }
    } // end of property ConfigFilename
  
    ///<summary>
    ///Read-only property that returns a reference to the internal XmlDocument.
    ///</summary>
    ///<Params>
    ///None.
    ///</Params>
    ///<returns type="XmlDocument">
    ///XmlDocument containing the configuration data.
    ///</returns>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration();
    ///myConfigFile.ConfigFilename = @"d:\Vha.Vss\UI\web.config";
    ///myConfigFile.Load();
    ///Console.WriteLine( myConfigFile.outerXml);
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public XmlDocument XmlDoc
    {
      get { return m_xmlDoc; }
    }
    ///<summary>
    ///Performs the disk IO to retrieve the XML data from the .config file.
    ///</summary>
    ///<Params>None
    ///</Params>
    ///<returns type="void ">
    ///</returns>
    ///<remarks>
    ///This function must be called to retrieve the data.  The 2 step creation of the object allows for setting
    ///of properties prior to the filesystem call.
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="VhaException" col2="If the ConfigFilename property has not been set an exception is thrown."/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Microsoft.eSupport.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration(@"d:\Vha.Vss\UI\web.config");
    ///myConfigFile.Load();
    ///Console.WriteLine( "\nHere is the configuration file specific information..." 
    ///  + "\n\tComponentName:\t\t {0}"
    ///  + "\n\tComponentInstanceName:\t {1}"
    ///  + "\n\tComponentVersion:\t {2}"
    ///  + "\n\tInstalledToPath:\t {3}"
    ///  + "\n\tInstalledFromPath:\t {4}"
    ///  + "\n\tConfigFilename:\n\t{5}",
    ///  myConfigFile["ComponentName"],
    ///  myConfigFile["ComponentInstanceName"],
    ///  myConfigFile["ComponentVersion"],
    ///  myConfigFile["InstalledToPath"],
    ///  myConfigFile["InstalledFromPath"],
    ///  myConfigFile["ConfigFilename"]);
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public void Load()
    {
      if (m_ConfigFilename == "")
      {
        VhaException.ThrowVhaException("Unable to load the configuration file because the ConfigFilename property has not been set.");
      }
      try
      {
        m_xmlDoc.Load( m_ConfigFilename );
      }
      catch (Exception e)
      {
        String Msg = String.Format("AppConfiguration.Load() failed on file [{0}]. Nested Exception info contains the details.", m_ConfigFilename);
        VhaException.ThrowAndLogException(300, Msg, this.myLog, e);
      }
    } // end of member Load()

    ///<summary>
    ///Performs the disk IO to retrieve the XML data from the .config file.
    ///</summary>
    ///<Params>
    ///<param type="String" name="fileName" Required="Mandatory">
    ///Application.config filename to be loaded and used by the AppConfiguration object.
    ///</param>
    ///</Params>
    ///<returns type="void ">
    ///</returns>
    ///<remarks>
    ///This function must be called to retrieve the data only when you did not use GetAppConfiguration().  
    ///The 2 step creation of the object allows for setting
    ///of properties prior to the filesystem call.
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="eSupportException" col2="If the ConfigFilename property has not been set an exception is thrown."/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration();
    ///myConfigFile.Load(@"d:\Vha.Vss\UI\web.config");
    ///Console.WriteLine( "\nHere is the configuration file specific information..." 
    ///  + "\n\tComponentName:\t\t {0}"
    ///  + "\n\tComponentInstanceName:\t {1}"
    ///  + "\n\tComponentVersion:\t {2}"
    ///  + "\n\tInstalledToPath:\t {3}"
    ///  + "\n\tInstalledFromPath:\t {4}"
    ///  + "\n\tConfigFilename:\n\t{5}",
    ///  myConfigFile["ComponentName"],
    ///  myConfigFile["ComponentInstanceName"],
    ///  myConfigFile["ComponentVersion"],
    ///  myConfigFile["InstalledToPath"],
    ///  myConfigFile["InstalledFromPath"],
    ///  myConfigFile["ConfigFilename"]);
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public void Load(string fileName)
    {
      m_ConfigFilename = fileName;
      this.Load();
    } // end of member Load()

    /// <summary>
    /// This is the indexer property implementation for retrieving configuration data.
    /// </summary>
    ///<Params>
    ///<param type="String" name="index" Required="Mandatory">
    ///String to be used to locate the specified ComponentAttribute item.
    ///</param>
    ///</Params>
    ///<returns type="String">
    ///Returns the value of the requested ComponentAttribute item.
    ///</returns>
    ///<remarks>
    ///This replaces the previously provide GetComponentAttribute() method.
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="eSupportException" col2="If the ConfigFilename property has not been set an exception is thrown."/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = new AppConfiguration();
    ///myConfigFile.Load(@"d:\Vha.Vss\UI\web.config");
    ///Console.WriteLine( "\nHere is the configuration file specific information..." 
    ///  + "\n\tComponentName:\t\t {0}"
    ///  + "\n\tComponentInstanceName:\t {1}"
    ///  + "\n\tComponentVersion:\t {2}"
    ///  + "\n\tInstalledToPath:\t {3}"
    ///  + "\n\tInstalledFromPath:\t {4}"
    ///  + "\n\tConfigFilename:\n\t{5}",
    ///  myConfigFile["ComponentName"],
    ///  myConfigFile["ComponentInstanceName"],
    ///  myConfigFile["ComponentVersion"],
    ///  myConfigFile["InstalledToPath"],
    ///  myConfigFile["InstalledFromPath"],
    ///  myConfigFile["ConfigFilename"]);
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public String this[String index]
    {
      get
      {
        // This will find the first entry in the hierarchy that contains the requested attribute
        XmlNode ComponentAttributes = m_xmlDoc.SelectSingleNode("//ComponentAttributes[@"+ index + "]");
        if (ComponentAttributes == null)
        {
          VhaException.ThrowVhaException("Attempt to retrieve the ComponentAttribute '" + index + "' failed because the <ComponentAttribute> tag in the XML file is either missing or there is more than one.");
        }
        return ComponentAttributes.Attributes.GetNamedItem(index).Value;
      }
      set
      {
        // This can only deal with a single level of hierarchy
        if (m_xmlDoc.SelectNodes("//ComponentAttributes[@"+ index + "]").Count != 1)
        {
          throw new VhaException("Setting ComponentAttributes is only allowed on existing items and in objects not containing a hierarchy of files.");
        }
        //  Now we know this will yield the single result we are expecting
        XmlNode ComponentAttributes = m_xmlDoc.SelectSingleNode("//ComponentAttributes");
        if (ComponentAttributes == null)
        {
          VhaException.ThrowVhaException("Attempt to set the ComponentAttribute '" + index + "' failed because the <ComponentAttribute> tag in the XML file is either missing or there is more than one.");
        }
        XmlNode newNode = m_xmlDoc.CreateAttribute(index);
        newNode.Value = value;
        ComponentAttributes.Attributes.SetNamedItem(newNode);
      }
    }


    ///<summary>
    ///Returns an object containing the database connection parameters to be used for accessing an VSS recognized 
    ///database.  Not to be confused with the VssConnection class that actually makes a network connection.
    ///</summary>
    ///<Params>
    ///<param type="String" name="DBConnectionName" Required="Mandatory">
    ///Logical name of the database connection that is to be searched for in the .config file.
    ///</param>
    ///</Params>
    ///<returns type="public DBConnection ">
    ///<para>
    ///The DBConnection object returned provides access to the Server, Database, UserId, Password, etc. data
    ///required to connect to a SQL database.
    ///</para>
    ///</returns>
    ///<remarks>
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="eSupportException" col2="If there are no &lt;DBConnection&gt; tags in the XML file an exception is thrown."/>
    ///<Row col1="eSupportException" col2="If the specified DBConnectionName is not found in the XML file an exception is thrown."/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Vha.Vss.Utility.Configuration;
    ///...
    ///AppConfiguration myConfigFile = AppConfiguration.GetAppConfiguration());
    ///DBConnection myDBConn = myConfigFile.GetDBConnection("eSupport");
    ///Console.WriteLine("Connection String: " + myDBConn.ConnectionString);
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="DBConnection"/>
    ///</References>
    public DBConnection GetDBConnection (String DBConnectionName)
    {
      // These may or may not exist
      XmlNode DBConnectionNode = this.GetConfigNode("//DBConnection[@DBConnectionName = '" + DBConnectionName + "']");
      if (DBConnectionNode == null)
      {
        VhaException.ThrowVhaException("Attempt to retrieve the DBConnection '" + DBConnectionName + "' failed because there are no <DBConnection> tags in the XML file with that name.");
      }
      return new DBConnection(DBConnectionNode);
    } // end of member GetDBConnection()

		/// <summary>
		/// Gets the XmlNode from the web.config specified by the XPath statement given
		/// </summary>
		/// <params>
		/// <param name="NodeXPath" type="System.String"/>
		/// </params>
		/// <returns type="System.Xml.XmlNode"/>
		public XmlNode GetConfigNode(String NodeXPath)
		{
			return m_xmlDoc.SelectSingleNode(NodeXPath);
		}

		/// <summary>
		/// Gets the DSN Configuration XmlNode.
		/// </summary>
		/// <param name="SectionGroupName"></param>
		/// <param name="DSNName"></param>
		/// <returns></returns>
		public XmlNode GetDSNConfigData (String SectionGroupName, String DSNName)
		{
			// These may or may not exist
			XmlNode DSNNode;
			if(DSNName == String.Empty)
							DSNNode = this.GetConfigNode("configuration/" + SectionGroupName + "/DSN");
			else
							DSNNode = this.GetConfigNode("configuration/" + SectionGroupName + "/DSN[@Name = '" + DSNName + "']");

			if (DSNNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the DSN '" + DSNName + "' failed because there are no <DSN> tags in the XML file with that name.");
			}
			return DSNNode;
		}
		/// <summary>
		/// Sets the DSN Configuration XmlNode
		/// </summary>
		/// <param name="SectionGroupName"></param>
		/// <param name="DSNName"></param>
		/// <param name="NodeName"></param>
		/// <param name="Data"></param>
		public void SetDSNConfigData (String SectionGroupName, String DSNName, String NodeName, String Data)
		{
			// These may or may not exist
			XmlNode DSNNode = m_xmlDoc.SelectSingleNode("configuration/" + SectionGroupName + "/DSN[@Name = '" + DSNName + "']");
			if (DSNNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the DSN Connection Data '" + DSNName + "' failed because there are no <DSN> tags in the XML file with that name.");
			}

			if(m_xmlDoc.SelectSingleNode("configuration/" + SectionGroupName + "/DSN[@Name = '" + DSNName + "']").Attributes.GetNamedItem(NodeName) == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Attribute ('" + NodeName + "') from Configuration Element ('" + DSNName + "') failed because there are no <DSN> tags in the XML file with that attribute name.");
			}
			else
			{
				m_xmlDoc.SelectSingleNode("configuration/" + SectionGroupName + "/DSN[@Name = '" + DSNName + "']").Attributes.GetNamedItem(NodeName).Value = Data;
			}
		}

		/// <summary>
		/// Gets the Database Configuration XmlNode.
		/// </summary>
		/// <param name="SectionGroupName"></param>
		/// <param name="DBConnectionName"></param>
		/// <returns></returns>
		public XmlNode GetDBConfigData (String SectionGroupName, String DBConnectionName)
		{
			// These may or may not exist
			XmlNode DBConnectionNode;
			if(DBConnectionName == String.Empty)
				DBConnectionNode = this.GetConfigNode("configuration/" + SectionGroupName + "/DBConnection");
			else
				DBConnectionNode = this.GetConfigNode("configuration/" + SectionGroupName + "/DBConnection[@Name = '" + DBConnectionName + "']");

			if (DBConnectionNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Database Connection '" + DBConnectionName + "' failed because there are no <DBConnection> tags in the XML file with that name.");
			}
			return DBConnectionNode;
		}

		/// <summary>
		/// Sets the Database Configuration XmlNode.
		/// </summary>
		/// <param name="SectionGroupName"></param>
		/// <param name="DBConnectionName"></param>
		/// <param name="NodeName"></param>
		/// <param name="Data"></param>
		public void SetDBConfigData (String SectionGroupName, String DBConnectionName, String NodeName, String Data)
		{
			// These may or may not exist
			XmlNode DBConnectionNode = m_xmlDoc.SelectSingleNode("configuration/" + SectionGroupName + "/DBConnection[@Name = '" + DBConnectionName + "']");
			if (DBConnectionNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Database Connection Data '" + DBConnectionName + "' failed because there are no <DBConnection> tags in the XML file with that name.");
			}

			if(m_xmlDoc.SelectSingleNode("configuration/" + SectionGroupName + "/DBConnection[@Name = '" + DBConnectionName + "']").Attributes.GetNamedItem(NodeName) == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Attribute ('" + NodeName + "') from Configuration Element ('" + DBConnectionName + "') failed because there are no <DBConnection> tags in the XML file with that attribute name.");
			}
			else
			{
				this.GetConfigNode("configuration/" + SectionGroupName + "/DBConnection[@Name = '" + DBConnectionName + "']").Attributes.GetNamedItem(NodeName).Value = Data;
			}
		}

		/// <summary>
		/// Gets the Application Level Configuration XmlNode.
		/// </summary>
		/// <param name="SectionGroupName"></param>
		/// <param name="ApplicationName"></param>
		/// <returns></returns>
		public XmlNode GetAppConfigData (String SectionGroupName, String ApplicationName)
		{
			// These may or may not exist
			XmlNode ApplicationNode;
			if(ApplicationName == String.Empty)
				ApplicationNode = this.GetConfigNode("configuration/" + SectionGroupName + "/Application");
			else
				ApplicationNode = this.GetConfigNode("configuration/" + SectionGroupName + "/Application[@Name = '" + ApplicationName + "']");

			if (ApplicationNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Application Data '" + ApplicationName + "' failed because there are no <Application> tags in the XML file with that name.");
			}
			return ApplicationNode;
		}
		/// <summary>
		/// Sets the Application Level Configuration XmlNode.
		/// </summary>
		/// <param name="ApplicationName"></param>
		/// <param name="NodeName"></param>
		/// <param name="Data"></param>
		public void SetAppConfigData (String ApplicationName, String NodeName, String Data)
		{
			// These may or may not exist
			XmlNode ApplicationNode = m_xmlDoc.SelectSingleNode("//Application[@Name = '" + ApplicationName + "']");
			if (ApplicationNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Application Data '" + ApplicationName + "' failed because there are no <Application> tags in the XML file with that name.");
			}

			if(this.GetConfigNode("//Application[@Name = '" + ApplicationName + "']").Attributes.GetNamedItem(NodeName) == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Attribute ('" + NodeName + "') from Configuration Element ('" + ApplicationName + "') failed because there are no <Application> tags in the XML file with that attribute name.");
			}else{
				this.GetConfigNode("//Application[@Name = '" + ApplicationName + "']").Attributes.GetNamedItem(NodeName).Value = Data;
			}
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="SectionGroupName"></param>
		/// <param name="ElementLocalName"></param>
		/// <param name="ElementAttributeName"></param>
		/// <param name="ElementAttributeValue"></param>
		/// <returns></returns>
		public XmlNode GetCustomConfigData(String SectionGroupName, String ElementLocalName, String ElementAttributeName, String ElementAttributeValue)
		{
			XmlNode CustomNode;
			String CustomXPath;
			if(ElementLocalName == String.Empty || ElementLocalName == null)
			{
				CustomXPath = "configuration/" + SectionGroupName + "/" + ElementLocalName;
				CustomNode = this.GetConfigNode(CustomXPath);
			}
			else
			{
				CustomXPath = "configuration/" + SectionGroupName + "/" + ElementLocalName + "[@" + ElementAttributeName + "='" + ElementAttributeValue + "']";
				CustomNode = this.GetConfigNode(CustomXPath);
			}
			if(CustomNode == null)
			{
				VhaException.ThrowVhaException("Attempt to retrieve the Custom Data from '" + ElementLocalName + "' failed because there are no <" + ElementLocalName + "> tags in the web.config file at XPath '" + CustomXPath + "'.");
			}
			return CustomNode;
		}


    /// <summary>
    /// This is the required implementation for the IConfigurationSectionHandler
    /// interface.  The URT configuration classes will call this member to enable
    /// custom handling of configuration data.  This class will return a fully
    /// constructed and ready to use AppConfiguration object.
    /// </summary>
    /// <Params>
    /// <param name="parent" type="Object">When present this object has an underlying 
    /// type of AppConfiguration.  It is not null when there is a hierarchy of 
    /// configuration sections in the URT probing path.</param>
    /// <param name="configContext">[To be supplied]</param>
    /// <param name="xmlNode">The XML node from the configuration file that 
    /// contains the custom data.</param>
    /// </Params>
    ///<returns type="public object">
    /// Returns a fully constructed and ready to use AppConfiguration object.
    /// You must typecast the return value.
    /// </returns>
    public object Create(Object parent, Object configContext, XmlNode xmlNode)
    {
      AppConfiguration parentObject = (AppConfiguration)parent;
		try
		{
			if (parentObject == null) 
			{
				//  first call for loading the first file found.  If there is a hierarchy
				//  the machine.config is first, app.config next, etc.
				m_xmlDoc.CreateXmlDeclaration("1.0", "UTF-8", ""); 
				XmlDocumentFragment frag = m_xmlDoc.CreateDocumentFragment();
				frag.InnerXml = "<Doc>" + xmlNode.OuterXml + "</Doc>";
				m_xmlDoc.AppendChild(frag);
			}
			else
			{
				//  Subsequent calls indicate a hierarchy so add new nodes to front of list
				//  so the XPath queries find them first.
				XmlDocumentFragment frag = m_xmlDoc.CreateDocumentFragment();
				frag.InnerXml = xmlNode.OuterXml;
				m_xmlDoc.DocumentElement.PrependChild(frag);
			}
		}
		finally
		{
		}
		return this;
	}

    /// <summary>
    /// Saves the currently loaded AppConfiguration object to the same file that
    /// was used to load it.  Call this API only when the Load() method was used
    /// to load object.
    /// </summary>
    /// <Params>None.
    /// </Params>
    ///<returns type="void">
    /// None.
    /// </returns>
    public void Save()
    {
      Save(m_ConfigFilename);
    }
    /// <summary>
    /// Saves the currently loaded AppConfiguration object to the designated
    /// filename.  
    /// </summary>
    /// <Params>
    /// <param name="filename" type="String">
    /// Fully qualified filename to save the file to.  Read-Only attributes (if present)
    /// will be removed prior to writing.
    /// </param>
    /// </Params>
    ///<returns type="void">
    /// None.
    /// </returns>
    public void Save(String filename)
    {
      if (filename == null || filename == "")
      {
        throw new VhaException("No filename was specified for the AppConfiguration object to save.");
      }
      if(File.Exists(filename)) File.SetAttributes(filename, FileAttributes.Normal);
      XmlTextWriter xmlWriter = new XmlTextWriter(filename, System.Text.Encoding.UTF8);
      xmlWriter.Formatting = Formatting.Indented;
      xmlWriter.Indentation = 2;
      m_xmlDoc.WriteContentTo(xmlWriter);
      xmlWriter.Close();
    } // end Save(fn)
  } // end of class AppConfiguration
} // end of namespace Vha.Vss.Utility.Configuration 
