using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Specialized;

namespace Vha.Vss.Utility
{
  /// <summary>
  ///Common class used throughout eSupport applications to make connections to the data sources.
  ///Currently this class aggregates the SqlConnection and SqlCommand classes and uses them to
  ///perform the actual connection and query.
  /// </summary>
  public class VhaConnection 
  {
    private SqlCommand m_SqlCommand;

    ///<summary>
    ///Creates an eSupportConnection object and intilizes the SqlConnection and SqlCommand members in
    ///preparation for running a stored procedure.  You will need to set the Connection.ConnectionString 
    ///property explicitly.
    ///</summary>
    ///<Params>None
    ///</Params>
    ///<remarks>
    ///</remarks>
    ///<References>
    /// <seealso cref="VhaConnection(String)"/>
    ///</References>
    public VhaConnection() 
    {
      InitializeComponent();
    }

    ///<summary>
    ///Creates an VhaConnection object and intilizes the SqlConnection and SqlCommand members in
    ///preparation for running a stored procedure.  Connection parameters are established by the passed in string.
    ///</summary>
    ///<Params>
    ///  <param type="System.String" name="strConnection" Required="">
    ///  Contains the connection string that contains the data source, database, user and password.
    ///  </param>
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///Use this construction to instantiate the class with a connection string.   
    ///</remarks>
    ///<Examples>
    /// <example language="C#">
    ///<code>
    ///string strConStr = "server=myserver;uid=myuser;pwd=mypassword;database=mydatabase";
    ///eSupportConnection objESupCon = new eSupportConnection(strConStr);
    ///</code>
    /// </example>
    ///</Examples>
    ///<References>
    /// <seealso cref="Command"/>
    ///</References>
    public VhaConnection(string strConnection)
    { 
      InitializeComponent();  // call this first - it creates the contained Connection/Command objects
      m_SqlCommand.Connection.ConnectionString = strConnection;
    }
    

    ///<summary>
    ///Initializes the internal SqlCommand and SqlConnection members.
    ///</summary>
    ///<returns type="private void ">
    ///</returns>
    ///<remarks>
    ///Sets the SqlCommand.CommandType to run stored procedures by default.  
    ///</remarks>
    private void InitializeComponent()
    {
      m_SqlCommand = new SqlCommand();
      m_SqlCommand.Connection = new SqlConnection();
    
      //  By default we assume that the type of call is a stored procedure.  If this is not
      //  the case, then the called must override this value accordingly 
      m_SqlCommand.CommandType = System.Data.CommandType.StoredProcedure;
    }

    ///<summary>
    ///Read-only propert returns the SqlConnection that is contained in the class.
    ///</summary>
    ///<remarks>
    ///The Connection is read-only but its properties can be modified.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///eSupportConnection myConn = new eSupportConnection();
    ///myConn.Connection.ConnectionTimeout = 15;
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="Command"/>
    ///</References>
    public SqlConnection Connection
    {
      get { return Command.Connection; } // Read-only property
    }


    ///<summary>
    ///Read-only propert returns the SqlCommand that is contained in the class.
    ///</summary>
    ///<remarks>
    ///The Command is read-only but its properties can be modified.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///eSupportConnection myConn = new eSupportConnection();
    ///myConn.Command.CommandType = SqlDbType.Text;
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="Connection"/>
    ///</References>
    public SqlCommand Command
    {
      get { return m_SqlCommand; } // Read-only property
    }


    ///<summary>
    ///Executes a SQL query and returns the results in the string reference passed in.  
    ///This method assumes the query will return XML content or only one row with one column of string type.
    ///</summary>
    ///<Params>
    ///<param type="ref string" name="xmlReturnValue" Required="">
    ///String passed in by caller to receive the query's output.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///<para>
    ///The method itself returns a void.  Query output is returned in the xmlReturnValue parameter.
    ///</para>
    ///</returns>
    ///<remarks>
    ///Only use this method for queries that return XML.  See the other overloads for versions that return a DataReader.
    ///All overloads that return strings will close the connection upon completion.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  String QueryResults="";
    ///  myDBConn.Command.CommandType = CommandType.Text;
    ///  myDBConn.Command.CommandText = "select top 1 Name from " + myDBConn.Connection.Database + "..sysobjects");
    ///  myDBConn.ExecuteSql(ref QueryResults);
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ExecuteSql(ref string,string)"/>
    /// <seealso cref="ExecuteSql(ref string,string,CommandType)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string,CommandType)"/>
    ///</References>
    public void ExecuteSql(ref string xmlReturnValue)
    {
      SqlDataReader dataReader = null;  // data Reader class for reading
      try
      {
        /* Execute the method that returns data reader */
        ExecuteSql(ref dataReader);
        
        // read the record set
        while (dataReader.Read()) 
        {
          xmlReturnValue += dataReader.GetString(0);  // get the first column in all rows of type string
        }    
        return;
      }
      catch(Exception ex)
      {
        throw ex;   //  if we have an exception, we have nothing to do, so escalate this to the caller 
      }     
      finally
      {
        /* Close the data reader */
        if (dataReader != null)
        {
          if (dataReader.IsClosed == false)
          {
            dataReader.Close();
          }
        }
        /* close the connection */
        if (Connection.State == System.Data.ConnectionState.Open)
        {
          Connection.Close();
        }
      }
    }

    ///<summary>
    ///Executes a SQL query of the type specified and returns the results in the string reference passed in.  
    ///This method assumes the query will return XML content or only one row with one column of string type.
    ///</summary>
    ///<Params>
    ///<param type="ref string" name="xmlReturnValue" Required="">
    ///String passed in by caller to receive the query's output.
    ///</param>
    ///<param type="string" name="sqlQuery" Required="">
    ///Query to be executed.  Can be any valid query.  Be sure to use the appropriate CommandType for the type
    ///of query passed in.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///<para>
    ///The method itself returns a void.  Query output is returned in the xmlReturnValue parameter.
    ///</para>
    ///</returns>
    ///<remarks>
    ///Only use this method for queries that return XML.  See the other overloads for versions that return a DataReader.
    ///All overloads that return strings will close the connection upon completion.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  String QueryResults="";
    ///  myDBConn.Command.CommandType = CommandType.Text;
    ///  myDBConn.ExecuteSql(ref QueryResults,
    ///    "select top 1 Name from " + myDBConn.Connection.Database + "..sysobjects");
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ExecuteSql(ref string)"/>
    /// <seealso cref="ExecuteSql(ref string,string,CommandType)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string,CommandType)"/>
    ///</References>
    public void ExecuteSql(ref string xmlReturnValue, string sqlQuery)
    {
      Command.CommandText = sqlQuery;
      ExecuteSql(ref xmlReturnValue);
    }


    ///<summary>
    ///Executes a SQL query of the type specified and returns the results in the string reference passed in.  
    ///This method assumes the query will return XML content or only one row with one column of string type.
    ///</summary>
    ///<Params>
    ///<param type="ref string" name="xmlReturnValue" Required="">
    ///String passed in by caller to receive the query's output.
    ///</param>
    ///<param type="string" name="sqlQuery" Required="">
    ///Query to be executed.  Can be any valid query.  Be sure to use the appropriate CommandType for the type
    ///of query passed in.
    ///</param>
    ///<param type="CommandType" name="commandType" Required="">
    ///Tells SQL Server what type of query has been passed in.  Use CommandType.Text for "select * from..." statements and 
    ///CommandType.StoredProcedure for stored procedures.  See SqlClient.CommandType documentation for complete
    ///details.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///<para>
    ///The method itself returns a void.  Query output is returned in the xmlReturnValue parameter.
    ///</para>
    ///</returns>
    ///<remarks>
    ///Only use this method for queries that return XML.  See the other overloads for versions that return a DataReader.
    ///All overloads that return strings will close the connection upon completion.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  String QueryResults="";
    ///  myDBConn.ExecuteSql(ref QueryResults,
    ///    "select top 1 Name from " + myDBConn.Connection.Database + "..sysobjects",
    ///    CommandType.Text);
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ExecuteSql(ref string)"/>
    /// <seealso cref="ExecuteSql(ref string,string)"/>
    /// <seealso cref="ExecuteSql(ref string,string,CommandType)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string,CommandType)"/>
    ///</References>
    public void ExecuteSql(ref string xmlReturnValue, string sqlQuery, CommandType commandType)
    {
      Command.CommandType = commandType;
      ExecuteSql(ref xmlReturnValue, sqlQuery);
    }
      
    //******** be suer to document that callers to this API must call eSUpportConnection.Connection.Close() when finished with their dataset.

    ///<summary>
    ///Executes a SQL query and returns the results in the SqlDataReaderpassed in.  
    ///</summary>
    ///<Params>
    ///<param type="ref SqlDataReader" name="dataReader" Required="">
    ///SqlDataReader passed in by caller to receive the query's output.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///<para>
    ///The method itself returns a void.  Query output is returned in the SqlDataReader parameter.
    ///</para>
    ///</returns>
    ///<remarks>
    ///Use this method for non-XML queries.  The database connection is not closed on any of the SqlDataReader
    ///methods because the SqlDataReader needs the connection to access the data.  The caller is obligated to 
    ///close the connection when all data has been consumed.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  SqlDataReader dataReader=null;
    ///  myDBConn.Command.CommandType = CommandType.Text;
    ///  myDBConn.Command.CommandText = "select top 1 Name from " + myDBConn.Connection.Database + "..sysobjects");
    ///  myDBConn.ExecuteSql(ref dataReader);
    ///  [do stuff with dataReader]
    ///  myDBConn.Connection.Close();
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ExecuteSql(ref string)"/>
    /// <seealso cref="ExecuteSql(ref string,string)"/>
    /// <seealso cref="ExecuteSql(ref string,string,CommandType)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string,CommandType)"/>
    ///</References>
    public void ExecuteSql(ref SqlDataReader dataReader)
    {
      /* If the query is empty, throw invalid operation exception. This is what SQLConnection does anyway */
      if (Command.CommandText.Length <= 0)
      {
        throw new System.InvalidOperationException("ExecuteSQL requires a sql query string to execute");
      }

      try
      {
        Connection.Open();  // open the connection
        dataReader = Command.ExecuteReader(); //execute the query
        return;       
      }
      catch(Exception ex)
      {
        /* In the case of exception we don't have to hold on to the connection. so close it */
        if (Connection.State == System.Data.ConnectionState.Open)
        {
          Connection.Close();
        }
        throw ex;
      }
      //  we don't have a finally block here.  Because we should release the resources
      //  only if there is an exception raised.  If the tx execution is successful, we should
      //  just return data reader.  Closing the connection will make the data reader invalid 
    }

    ///<summary>
    ///Executes a SQL query and returns the results in the SqlDataReaderpassed in.  
    ///</summary>
    ///<Params>
    ///<param type="ref SqlDataReader" name="dataReader" Required="">
    ///SqlDataReader passed in by caller to receive the query's output.
    ///</param>
    ///<param type="string" name="sqlQuery" Required="">
    ///Query to be executed.  Can be any valid query.  Be sure to use the appropriate CommandType for the type
    ///of query passed in.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///<para>
    ///The method itself returns a void.  Query output is returned in the SqlDataReader parameter.
    ///</para>
    ///</returns>
    ///<remarks>
    ///Use this method for non-XML queries.  The database connection is not closed on any of the SqlDataReader
    ///methods because the SqlDataReader needs the connection to access the data.  The caller is obligated to 
    ///close the connection when all data has been consumed.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  SqlDataReader dataReader=null;
    ///  myDBConn.Command.CommandType = CommandType.Text;
    ///  myDBConn.ExecuteSql(ref dataReader,
    ///    "select top 1 Name from " + myDBConn.Connection.Database + "..sysobjects");
    ///  [do stuff with dataReader]
    ///  myDBConn.Connection.Close();
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ExecuteSql(ref string)"/>
    /// <seealso cref="ExecuteSql(ref string,string)"/>
    /// <seealso cref="ExecuteSql(ref string,string,CommandType)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string,CommandType)"/>
    ///</References>
    public void ExecuteSql(ref SqlDataReader dataReader, string sqlQuery)
    {
      Command.CommandText = sqlQuery;
      ExecuteSql(ref dataReader);
    } // end of ExecuteSql(dataReader, sqlQuery)


    ///<summary>
    ///Executes a SQL query of the type specified and returns the results in the SqlDataReaderpassed in.  
    ///</summary>
    ///<Params>
    ///<param type="ref SqlDataReader" name="dataReader" Required="">
    ///SqlDataReader passed in by caller to receive the query's output.
    ///</param>
    ///<param type="string" name="sqlQuery" Required="">
    ///Query to be executed.  Can be any valid query.  Be sure to use the appropriate CommandType for the type
    ///of query passed in.
    ///</param>
    ///<param type="CommandType" name="commandType" Required="">
    ///Tells SQL Server what type of query has been passed in.  Use CommandType.Text for "select * from..." statements and 
    ///CommandType.StoredProcedure for stored procedures.  See SqlClient.CommandType documentation for complete
    ///details.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///<para>
    ///The method itself returns a void.  Query output is returned in the SqlDataReader parameter.
    ///</para>
    ///</returns>
    ///<remarks>
    ///Use this method for non-XML queries.  The database connection is not closed on any of the SqlDataReader
    ///methods because the SqlDataReader needs the connection to access the data.  The caller is obligated to 
    ///close the connection when all data has been consumed.
    ///</remarks>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  SqlDataReader dataReader=null;
    ///  myDBConn.ExecuteSql(ref dataReader,
    ///    "select top 1 Name from " + myDBConn.Connection.Database + "..sysobjects",
    ///    CommandType.Text);
    ///  [do stuff with dataReader]
    ///  myDBConn.Connection.Close();
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ExecuteSql(ref string)"/>
    /// <seealso cref="ExecuteSql(ref string,string)"/>
    /// <seealso cref="ExecuteSql(ref string,string,CommandType)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader)"/>
    /// <seealso cref="ExecuteSql(ref SqlDataReader,string)"/>
    ///</References>
    public void ExecuteSql(ref SqlDataReader dataReader, string sqlQuery, CommandType commandType)
    {
      Command.CommandType = commandType;
      ExecuteSql(ref dataReader, sqlQuery);
    }


    ///<summary>
    ///Functions to add parameters for the stored procedure.  This adds parameter values to
    ///the SqlCommand object used by the class and performs all necessary character escaping for ANSI compliance.
    ///</summary>
    ///<Params>
    ///<param type="string" name="name" Required="">
    ///Name of the parameter being added.
    ///</param>
    ///<param type="int" name="size" Required="">
    ///Length of the string value being passed in.
    ///</param>
    ///<param type="string" name="value" Required="">
    ///String value of the parameter identified.
    ///</param>
    ///<param type="SqlDbType" name="dataType" Required="">
    ///Data type of the parameter being set.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///</returns>
    ///<remarks>
    ///This is a convenience function that leverages the .NET Command functionality that takes care of the 
    ///single quote/double quote conversions and other things that make a parameter value safe.
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="eSupportException" col2=" [reason an exception would be thrown] "/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  SqlDataReader dataReader=null;
    ///  myDBConn.AddCommandParameter("FirstName", 60, "Herman", SqlDbType.NVarChar);
    ///  myDBConn.ExecuteSql(ref dataReader, "MyStoredProc");
    ///  [do stuff with dataReader]
    ///  myDBConn.Connection.Close();
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="AddCommandParameter(string,int,SqlDbType)"/>
    /// <seealso cref="AddCommandParameter(string,bool,SqlDbType)"/>
    ///</References>
    public void AddCommandParameter(string name, int size, string value, SqlDbType dataType)
    {
      Command.Parameters.Add (new System.Data.SqlClient.SqlParameter(name, dataType, size));
      if (value.Length == 0) 
      {
        Command.Parameters[name].Value = null; 
      }
      else
      {
        Command.Parameters[name].Value = value;
      }
    } // end AddCommandParameter


    ///<summary>
    ///Functions to add parameters for the stored procedure.  This adds parameter values to
    ///the SqlCommand object used by the class and performs all necessary character escaping for ANSI compliance.
    ///</summary>
    ///<Params>
    ///<param type="string" name="name" Required="">
    ///Name of the parameter being added.
    ///</param>
    ///<param type="int" name="iValue" Required="">
    ///Numeric integer value to be used by the stored procedure.
    ///</param>
    ///<param type="SqlDbType" name="dataType" Required="">
    ///Data type of the parameter being set.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///</returns>
    ///<remarks>
    ///This is a convenience function that leverages the .NET Command functionality that takes care of the 
    ///single quote/double quote conversions and other things that make a parameter value safe.
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="eSupportException" col2=" [reason an exception would be thrown] "/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  SqlDataReader dataReader=null;
    ///  myDBConn.AddCommandParameter("RecId", 12, SqlDbType.Int);
    ///  myDBConn.ExecuteSql(ref dataReader, "MyStoredProc");
    ///  [do stuff with dataReader]
    ///  myDBConn.Connection.Close();
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="AddCommandParameter(string,int,string,SqlDbType)"/>
    /// <seealso cref="AddCommandParameter(string,int,SqlDbType)"/>
    /// <seealso cref="AddCommandParameter(string,bool,SqlDbType)"/>
    ///</References>
    public void AddCommandParameter(string name, int iValue, SqlDbType dataType)
    {
      Command.Parameters.Add (new SqlParameter(name, dataType));
      if (iValue < 0) 
      {
        Command.Parameters[name].Value = null; 
      }
      else
      {
        Command.Parameters[name].Value = iValue;
      }
    } // end AddCommandParameter


    ///<summary>
    ///Functions to add parameters for the stored procedure.  This adds parameter values to
    ///the SqlCommand object used by the class and performs all necessary character escaping for ANSI compliance.
    ///</summary>
    ///<Params>
    ///<param type="string" name="name" Required="">
    ///Name of the parameter being added.
    ///</param>
    ///<param type="bool" name="bValue" Required="">
    ///Boolean value to be passed to the stored procedure.
    ///</param>
    ///<param type="SqlDbType" name="dataType" Required="">
    ///Data type of the parameter being set.
    ///</param>
    ///</Params>
    ///<returns type="public void ">
    ///</returns>
    ///<remarks>
    ///This is a convenience function that leverages the .NET Command functionality that takes care of the 
    ///single quote/double quote conversions and other things that make a parameter value safe.
    ///</remarks>
    ///<Exceptions>
    ///<Table col1="Exception" col2="Description">
    ///<Row col1="eSupportException" col2=" [reason an exception would be thrown] "/>
    ///</Table>
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///private void RunDBTest(String DBConnectionName, Component myComponent)
    ///{
    ///  //  Test the DBConnection object...
    ///  DBConnection myDBConn = myComponent.GetDBConnection(DBConnectionName);
    ///  Console.WriteLine("Connection String: " + myDBConn.Connection.ConnectionString);
    ///  SqlDataReader dataReader=null;
    ///  myDBConn.AddCommandParameter("MyFlag", true, SqlDbType.Bit);
    ///  myDBConn.ExecuteSql(ref dataReader, "MyStoredProc");
    ///  [do stuff with dataReader]
    ///  myDBConn.Connection.Close();
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="AddCommandParameter(string,int,string,SqlDbType)"/>
    /// <seealso cref="AddCommandParameter(string,int,SqlDbType)"/>
    ///</References>
    public void AddCommandParameter(string name, bool bValue, SqlDbType dataType)
    {
      Command.Parameters.Add (new SqlParameter(name, dataType));
      Command.Parameters[name].Value = bValue;
    } // end AddCommandParameter

    /// <summary>
    /// Static method that accepts a SQL connection string as input and parses out
    /// each component into an array of string.  This function deals with the encrypted 
    /// passwords that would otherwise confuse a parser.
    /// </summary>
    /// <Params>
    /// <param name="connectionString"></param>
    /// </Params>
    /// <returns type="String[]">
    /// Returns a string array containing the parsed values.  The even numbered 
    /// array positions represent the name of the connection attribute.  The odd number
    /// array positions represent the value of the connection attribute.
    /// </returns>
    public static String[] GetConnectionStringAsArray(String connectionString)
    {
      String savePwd = "";
      int first, last=-1;

      //  All of this is just to keep encrypted passwords from screwing up the 
      //  splitting of an otherwise valid connection string.  The "=" equal signs
      //  really cause problems.  The plan is just to find the password, save it off
      //  and replace it with "bogus".  We use the saved off password later.
      connectionString = connectionString.TrimEnd(new char[] {';'});  // make sure it doesn't end with a semicolon.
      first = connectionString.IndexOf("Password=[");

      if (first>=0) last = connectionString.IndexOf("];", first);
      if (last > 0)
      {
        first += 9;
        savePwd = connectionString.Substring(first, last-first+1);
        connectionString = connectionString.Replace(savePwd, "bogus");
      }
      Char[] Delimiters = {';','='};
      String[] tmpArray = connectionString.Split(Delimiters);
      int i=0;
      while(savePwd!="" && i<tmpArray.Length)
      {
        if (tmpArray[i] == "Password")
        {
          tmpArray[i+1] = savePwd;
          savePwd = "";
        }
        i+=2;
      }
      return tmpArray;
    } // end GetConnectionStringAsArray()
    
    /// <summary>
    /// Static method that accepts a SQL connection string as input and parses out
    /// each component into a StringDictionary.  This function deals with the encrypted 
    /// passwords that would otherwise confuse a parser.
    /// </summary>
    /// <Params>
    /// <param name="connectionString">
    /// Standard SQL connection string with entries delimited by semicolons and name/value
    /// pairs seperated by equal signs.
    /// </param>
    /// </Params>
    /// <returns type="StringDictionary">
    /// Returns a StringDictionary containing the parsed values.  The key of the 
    /// dictionary is the name of the connection attribute.  
    /// </returns>
    /// <remarks>
    /// Note that the use of the StringDictionary converts all key values to lowercase.
    /// </remarks>
    public static StringDictionary GetConnectionStringAsDictionary(String connectionString)
    {
      StringDictionary tmpDict = new  StringDictionary();
      String[] tmpArray = GetConnectionStringAsArray(connectionString);
      for(int i=0; i<tmpArray.Length ; i+=2)
      {
        tmpDict.Add(tmpArray[i], tmpArray[i+1]);
      }
      return tmpDict;
    } // end GetConnectionStringAsDictionary


    /// <summary>
    /// Static method that accepts a StringDictionary as input and constructs a SQL
    /// connection string from the name/value pairs.  This is the opposite of 
    /// the GetConnectionStringAsDictionary.
    /// </summary>
    /// <Params>
    /// <param type="StringDictionary" name="theDictionary">
    /// StringDictionary with name/value pairs to use for constructing a standard
    /// SQL connection string with entries delimited by semicolons and name/value
    /// pairs seperated by equal signs.
    /// </param>
    /// </Params>
    /// <returns type="String">
    /// Returns a SQL connection string with entries delimited by semicolons and name/value
    /// pairs seperated by equal signs.
    /// </returns>
    /// <remarks>
    /// Note that the use of the StringDictionary converts all key values to lowercase.
    /// </remarks>
    public static String GetConnectionStringFromDictionary(StringDictionary theDictionary)
    {
      String newConnectionString="";
      if (theDictionary.Count>0)
      {
        foreach(String key in theDictionary.Keys)
        {
          newConnectionString += key + "=" + theDictionary[key] + ";" ;
        }
        // make sure it doesn't end with a semicolon.
        newConnectionString = newConnectionString.Remove(newConnectionString.Length-1,1);
      }
      return newConnectionString;
    }

  } // end of class eSupportConnection

} // end of namespace
