
namespace Vha.Vss.Utility
{
  using System;

  /// <summary>
  ///  The Exception class is a derivation of the System.Exception and extends the base class
  ///  by adding some custom properties for use by the Vha application.  Try/catch blocks should be
  ///  written to catch System.Exceptions and internally detect the class name actually caught.  
  /// </summary>
  /// 
  public class VhaException : System.ApplicationException
  {
    ///<summary>
    ///This overloaded static member function will create and throw an VhaException object.
    ///The function will set the Exception.Message property to the value passed in.
    ///This is the simplest way for an application to throw a VhaException.
    ///</summary>
    ///<Params>
    ///  <param type="System.String" name="Message" Required="">
    ///  Contains the descriptive text that will be placed into the exception and describes the error condition.
    ///  </param>
    ///</Params>
    ///<returns type="void ">
    ///</returns>
    ///<remarks>
    ///<para>
    ///This function is used primarily by the tier 2 components.  Applications have little reason for throwing VhaExceptions.
    ///</para>
    ///<para>
    ///If everything works successfully then an exception is thrown and a return value 
    ///would be meaningless.  If not successful, the exception that prevented the creation
    ///of the VhaException will be thrown.
    ///</para>
    ///</remarks>
    ///<Examples>
    ///	<example language="C#">
    ///<code>
    ///Using System;
    ///if (GetDBConnection() == Failed) {
    ///  ThrowVhaException(
    ///      "Failed to obtain a database connection from the connection pool.");
    ///}
    ///</code>
    ///	</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="VhaException"/>
    ///</References>
	  static public void ThrowVhaException(System.String Message) 
    {
		  throw new VhaException (Message);
	  }


    ///<summary>
    ///This overloaded static member function will create and throw an VhaException object.
    ///The function will set the Exception.Message property to the value passed in.  The Exception object
    ///passed in will be set into the InnerException property of the VhaException.
    ///This API is not typically used since normal standards would dictate the original exception be passed
    ///on to the caller.
    ///</summary>
    ///<Params>
    ///  <param type="System.String" name="Message" Required="">
    ///  Contains the descriptive text that will be placed into the exception and describes the error condition.
    ///  </param>
    ///  <param type="System.Exception" name="e" Required="">
    ///  Contains the System.Exception (or valid derivation) that has been thrown.  This exception will be
    ///  placed into the InnerException property of the VhaException.
    ///  </param>
    ///</Params>
    ///<returns type="void ">
    ///</returns>
    ///<remarks>
    ///<para>
    ///This function is used primarily by the tier 2 components.  Applications have little reason for throwing VhaExceptions.
    ///</para>
    ///<para>
    ///If everything works successfully then an exception is thrown and a return value 
    ///would be meaningless.  If not successful, the exception that prevented the creation
    ///of the VhaException will be thrown.
    ///</para>
    ///</remarks>
    ///<Examples>
    ///	<example language="C#">
    ///<code>
    ///Using System;
    ///try {
    ///   int rc = SomeBadAPICall();	// assume some exception gets thrown here
    ///}
    ///catch(System.Exception e) {
    ///   ThrowVhaException("Caught exception in Foo().", e);
    ///}
    ///</code>
    ///	</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="VhaException"/>
    ///</References>
	  static public void ThrowVhaException(System.String Message, System.Exception e) 
    {
		  throw new VhaException (Message, e);
	  }


    ///<summary>
    ///This static member is nearly identical to ThrowVhaException(System.String) with the one exception
    ///that it will first write to the eSupport logging utility before throwing the exception.  Once the 
    ///VhaException is constructed the VhaLog.Write(e) method is called then the exception is thrown.
    ///</summary>
    ///<Params>
    ///  <param type="System.Int" name="code" Required="">
    ///  Contains the Error Code if any exists.
    ///  </param>
    ///  <param type="System.String" name="Message" Required="">
    ///  Contains the descriptive text that will be placed into the exception and describes the error condition.
    ///  </param>
    ///  <param type="VhaLog" name="LogObject" Required="">
    ///  A valid instantiation of the VhaLog object.  This object will be used to record the event middle-tier component.
    ///  </param>
    ///</Params>
    ///<returns type="void ">
    ///</returns>
    ///<remarks>
    ///This API is used by the tier 2 transactions as a convenience function and is not intended for general
    ///application use.
    ///</remarks>
    ///<Examples>
    ///	<example language="C#">
    ///<code>
    ///Using System;
    ///int rc = SomeBadAPICall();
    ///if (rc != 0) {
    ///   ThrowAndLogException("SomeBadAPICall() didn't work.", myLog);
    ///}
    ///</code>
    ///	</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ThrowAndLogException(int, System.String, VhaLog, System.Exception)"/>
    ///</References>
	  static public void ThrowAndLogException(int code, System.String Message, VhaLog LogObject) 
    {
		  VhaException myException = new VhaException (Message);
			Vha.Vss.Utility.VhaLog.WriteLog("Vha.Vss", "Application", Message);
		  throw myException;
	  }


    ///<summary>
    ///This static member is nearly identical to ThrowVhaException(System.String, System.Exception) with the one exception
    ///that it will first write to the eSupport logging utility before throwing the exception.  Once the 
    ///VhaException is constructed the VhaLog.Write(e) method is called then the passed in exception is thrown.
    ///</summary>
    ///<Params>
    ///  <param type="System.Int" name="code" Required="">
    ///  Contains the Error Code if any exists.
    ///  </param>
    ///  <param type="System.String" name="Message" Required="">
    ///  Contains the descriptive text that will be placed into the exception and describes the error condition.
    ///  </param>
    ///  <param type="VhaLog" name="LogObject" Required="">
    ///  A valid instantiation of the VhaLog object.  This object will be used to record the event middle-tier component.
    ///  </param>
    ///  <param type="System.Exception" name="e" Required="">
    ///  Contains the System.Exception (or valid derivation) that has been thrown.  This exception will be
    ///  placed into the InnerException property of the VhaException.
    ///  </param>
    ///</Params>
    ///<returns type="void ">
    ///</returns>
    ///<remarks>
    ///<para>This API is used by the tier 2 transactions as a convenience function and is not intended for general application use.</para>
    ///<para>Note that this API does NOT throw an VhaException.  It throws the exception that is passed in.</para>
    ///</remarks>
    ///<Examples>
    ///	<example language="C#">
    ///<code>
    ///Using System;
    ///try {
    ///   int rc = SomeBadAPICall();	// assume some exception gets thrown here
    ///}
    ///catch(System.Exception e) {
    ///   VhaLog myLog = new VhaLog("TransactionLayer");
    ///   ThrowAndLogException("Caught exception in Foo().", e, myLog);
    ///}
    ///</code>
    ///	</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="ThrowAndLogException(int, System.String, VhaLog)"/>
    ///</References>
	  static public void ThrowAndLogException(int code, System.String Message, VhaLog LogObject, System.Exception e) 
    {
			VhaLog.WriteLog("Vha.Vss", "Application", Message, System.Diagnostics.EventLogEntryType.Error);
		  throw e;
	  }

      private VhaException() {}	// Don't allow construction without the required parameters


    ///<summary>
    ///Constructor that initializes the base class System.Exception read-only Message property.
    ///</summary>
    ///<Params>
    ///  <param type="System.String" name="Message" Required="">
    ///  Contains the descriptive text that will be placed into the exception and describes the error condition.
    ///  </param>
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///The Message property can only be set during construction.  It is a read-only property.
    ///</remarks>
    ///<Examples>
    ///	<example language="C#">
    ///<code>
    ///throw new VhaException("This is the exception Message.");
    ///</code>
    ///	</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="VhaException(System.String, System.Exception)"/>
    ///</References>
	  public VhaException(System.String Message) : base(Message)
    {
    }


    ///<summary>
    ///Constructor that initializes the base class System.Exception read-only Message property.
    ///</summary>
    ///<Params>
    ///  <param type="System.String" name="Message" Required="">
    ///  Contains the descriptive text that will be placed into the exception and describes the error condition.
    ///  </param>
    ///  <param type="System.Exception" name="e" Required="">
    ///  Contains the exception object that is to be placed in the InnerException property of the exception.
    ///  </param>
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///<para>The Message property can only be set during construction.  It is a read-only property.</para>
    ///<para>The InnerException property can only be set during construction.  It is a read-only property.</para>
    ///</remarks>
    ///<Examples>
    ///	<example language="C#">
    ///<code>
    ///Using System;
    ///try {
    ///   int rc = SomeBadAPICall();	// assume some exception gets thrown here
    ///}
    ///catch(System.Exception e) {
    ///   throw new VhaException ("Caught the enclosed exception in Foo().", e);
    ///}
    ///</code>
    ///	</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="VhaException(System.String)"/>
    ///</References>
	  public VhaException(System.String Message, System.Exception e) : base(Message, e)
    {
    }
  }
}