namespace Vha.Vss.Utility
{
  using System;
  using System.IO;
  using System.Xml;
  using System.Xml.XPath;
  using System.Xml.Xsl;
  using Vha.Vss.Utility;
  /// <summary>
  /// This class has several methods used to do transformations on XML/XSLT files.
  /// </summary>
  public class XmlTransformer
  {
    /// <summary>
    /// Standard class for obtaining the source for XML data used in the XmlTransform.  See the SDK
    /// for more information about the particular properties of XmlDataSource.
    /// </summary>
    public XmlDataSource XmlDataSourceParms;
    /// <summary>
    /// Standard class for obtaining the source for XSLT data used in the XmlTransform.  See the SDK
    /// for more information about the particular properties of XmlDataSource.
    /// </summary>
    public XslDataSource XslDataSourceParms;

    //  Provided for an application if they want it
    private XsltArgumentList m_XsltArguments;

    // internal use only. Not exposed through a property.
    private XPathNavigator   m_XmlDocNavigator; 

    /// <summary>
    /// Default Constructor.  Initializes the internal objects.
    /// </summary>
    public XmlTransformer()
    {
      m_XsltArguments    = new XsltArgumentList();
      XslDataSourceParms = new XslDataSource();
      XmlDataSourceParms = new XmlDataSource();
    }
    

    /// <summary>
    /// This is the XsltArgumentsList object that will be passed to the 
    /// URT transform methods.  Applications should add the parameters
    /// required to this object.
    /// </summary>
    public XsltArgumentList XsltArguments
    {
      get { return m_XsltArguments; }
      set { m_XsltArguments  = value; }
    }


    ///<summary>
    /// This Transform method takes a Stream and passes the transform result 
    /// to the Stream.
    ///</summary>
    ///<Params>
    /// <param name="outputStream" Type="System.IO.Stream">
    /// Any stream object which will be used to output the transformation text.
    /// </param>
    ///</Params>
    ///<returns type="public void">
    ///The method itself returns void.  Transformation output is returned
    ///in the Stream object passed in.
    ///</returns>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///XmlTransformer oTransformer = new XmlTransformer();
    ///oTransformer.XmlSourceFilename = "http://doc.xml";
    ///oTransformer.XslSourceFilename = "http://trans.xslt";
    ///oTransformer.Transform(Response.Output);
    ///</code>
    /// </example>
    ///</Examples>
    public void Transform(Stream outputStream)
    {
      try
      {
        InitializeDocuments();    // always call this prior to transforming
        XslDataSourceParms.XslTransformObject.Transform(m_XmlDocNavigator, m_XsltArguments, outputStream);
      }
      catch(Exception ex)
      {
        String msg = "Caught and exception in XmlTransformer.Transform(XmlDocument):\n" + VhaLog.BuildExceptionDetails(ex);
        msg += ToString();
        VhaLog.WriteLog("Vha.Vss.Utility", "Application", msg);
        throw;  // pass this back to the caller - a WebControl will catch a page trace
      }
    }


    /// <summary>
    /// This Transform method takes an XmlDocument and returns its results to the
    /// to the XmlDocument ref.
    /// </summary>
    /// <Params>
    ///   <param name="outputDoc" type="XmlDocument">
    ///   XmlDocument object which is to receive the output from the Transform.
    ///   </param>
    /// </Params>
    ///<returns type="public void">
    ///The method itself returns void.  Transformation output is returned
    ///in the XmlDocument object passed in.
    ///</returns>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///XmlTransformer oTransformer = new XmlTransformer();
    ///oTransformer.XmlSourceFilename = "http://doc.xml";
    ///oTransformer.XslSourceFilename = "http://trans.xslt";
    ///XmlDocument XmlDoc = new XmlDocument();
    ///oTransformer.Transform(XmlDoc);
    ///</code>
    ///</example>
    ///</Examples>
    public void Transform(XmlDocument outputDoc)
    { 
      try
      {
        InitializeDocuments();    // always call this prior to transforming
        outputDoc.Load(XslDataSourceParms.XslTransformObject.Transform(m_XmlDocNavigator, m_XsltArguments));
      }
      catch(Exception ex)
      {
        String msg = "Caught and exception in XmlTransformer.Transform(XmlDocument):\n" + VhaLog.BuildExceptionDetails(ex);
        msg += ToString();
				VhaLog.WriteLog("Vha.Vss.Utility", "Application", msg);
        throw;  // pass this back to the caller - a WebControl will catch a page trace
      }
    }

    /// <summary>
    /// This method uses the previously set properties and returns the results of the Transform as a string.
    /// </summary>
    /// <Params>
    ///None.
    /// </Params>
    ///<returns type="public String">
    ///Returns a string containing the results of the transform.
    ///</returns>
    ///<remarks/>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///XmlTransformer oTransformer = new XmlTransformer();
    ///oTransformer.XmlSourceFilename = "http://doc.xml";
    ///oTransformer.XslSourceFilename = "http://trans.xslt";
    ///myXmlTarget.innerHtml = oTransformer.Transform();
    ///</code>
    ///</example>
    ///</Examples>
    ///<References/>
    public String Transform()
    {
      //  Get a temporary stream object and transform into it.
      //  Use a stream instead of XmlDocument so that we don't force the transform output to be valid XML.
      MemoryStream stream = new MemoryStream();
      Transform(stream);

      // Return the transform output in the passed in ref String
      stream.Seek(0, SeekOrigin.Begin);   // set it to the beginning
      StreamReader sr = new StreamReader(stream);
      return sr.ReadToEnd();
    }

    /// <summary>
    /// This method accepts filenames and will load and transform based on them.
    /// Results of the Transform are returned as a string.
    /// </summary>
    /// <Params>
    ///   <param name="xmlSourceFilename" type="String">
    ///   Filename used to load the XmlDocument object which is to be transformed.
    ///   </param>
    ///   <param name="xslSourceFilename" type="String">
    ///   Filename used to load the XslTransform object which is to be 
    ///   used to perform the transform.
    ///   </param>
    /// </Params>
    ///<returns type="public String">
    ///Returns a string containing the results of the transform.
    ///</returns>
    ///<remarks/>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///XmlTransformer oTransformer = new XmlTransformer();
    ///myXmlTarget.innerHtml = oTransformer.Transform("http://doc.xml", "http://trans.xslt");
    ///</code>
    ///</example>
    ///</Examples>
    ///<References/>
    public String Transform(String xmlSourceFilename, String xslSourceFilename)
    {
      //  Set the public properties so the documents get wiped.
      XmlDataSourceParms.Filename = xmlSourceFilename;
      XslDataSourceParms.Filename = xslSourceFilename;
      return Transform();
    }

    /// <summary>
    /// This method accepts filenames and will load and transform based on them.
    /// Results of the Transform are returned as an XmlDocument.
    /// </summary>
    /// <Params>
    ///   <param name="outputDoc" type="XmlDocument">
    ///   XmlDocument object which is to receive the output from the Transform.
    ///   </param>
    ///   <param name="xmlSourceFilename" type="String">
    ///   Filename used to load the XmlDocument object which is to be transformed.
    ///   </param>
    ///   <param name="xslSourceFilename" type="String">
    ///   Filename used to load the XslTransform object which is to be 
    ///   used to perform the transform.
    ///   </param>
    /// </Params>
    ///<returns type="public void">
    ///The method itself returns void.  Transformation output is returned
    ///in the XmlDocument object passed in.
    ///</returns>
    ///<remarks/>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///XmlTransformer oTransformer = new XmlTransformer();
    ///XmlDocument XmlDoc = new XmlDocument();
    ///oTransformer.Transform(XmlDoc, "http://doc.xml", "http://trans.xslt");
    ///</code>
    ///</example>
    ///</Examples>
    ///<References/>
    public void Transform(XmlDocument outputDoc, String xmlSourceFilename, String xslSourceFilename)
    {
      //  Set the public properties so the documents get wiped.
      XmlDataSourceParms.Filename = xmlSourceFilename;
      XslDataSourceParms.Filename = xslSourceFilename;
      Transform(outputDoc);
    }

    /// <summary>
    /// This method accepts filenames and will load and transform based on them.
    /// Results of the Transform are returned as a Stream (such as the 
    /// standard ASP+ Response.Outpu stream).
    /// </summary>
    /// <Params>
    ///   <param name="outputStream" type="Stream">
    ///   Stream object which is to receive the output from the Transform.
    ///   </param>
    ///   <param name="xmlSourceFilename" type="String">
    ///   Filename used to load the XmlDocument object which is to be transformed.
    ///   </param>
    ///   <param name="xslSourceFilename" type="String">
    ///   Filename used to load the XslTransform object which is to be 
    ///   used to perform the transform.
    ///   </param>
    /// </Params>
    ///<returns type="public void">
    ///The method itself returns void.  Transformation output is returned
    ///in the Stream object passed in.
    ///</returns>
    ///<remarks/>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///XmlTransformer oTransformer = new XmlTransformer();
    ///oTransformer.Transform(Response.Output, "http://doc.xml", "http://trans.xslt");
    ///</code>
    ///</example>
    ///</Examples>
    ///<References/>
    public void Transform(Stream outputStream, String xmlSourceFilename, String xslSourceFilename)
    {
      //  Set the public properties so the documents get wiped.
      XmlDataSourceParms.Filename = xmlSourceFilename;
      XslDataSourceParms.Filename = xslSourceFilename;
      Transform(outputStream);
    }

    /// <summary>
    /// Internal private method that takes care of ensuring the XmlDocument, XslTransform and XsltNavigator 
    /// objects are all created and loaded.  The function will use objects if they exist
    /// and if not will create and load them from the filename properties.  This 
    /// function throws an VhaException when neither the object or filename properties exist.
    /// </summary>
    private void InitializeDocuments()
    {
      XmlDataSourceParms.InitializeDocumentObject();
      XslDataSourceParms.InitializeDocumentObject();
      m_XmlDocNavigator = ((IXPathNavigable)XmlDataSourceParms.XmlDocumentObject).CreateNavigator();
    } // end of InitializeDocuments method

    /// <summary>
    /// Returns a string representation of the object and all of its properties.
    /// </summary>
    /// <returns>String containing the contents of the objects properties.</returns>
    public override String ToString()
    {
      String Output = base.ToString() + "\n";
      try
      {
        Output += this.XmlDataSourceParms.ToString();
        Output += this.XslDataSourceParms.ToString();
      }
      catch(Exception ex)
      {
        Output += "XmlTransformer data not available due to exception: " + ex.Message;
      }
      return Output;
    }
  } // end XmlTransformer class
} // end namespace
