#define msSql

#region Section: Using Declaration

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Security.Principal;
using System.Threading;

using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;
using Vha.Vss.Utility;

using si = Vha.Vtk.Kernel.Common.ReturnStatusIndicator;

#endregion

namespace Vha.Vtk.Kernel.Common
{
	/// <summary>
	/// Contains properties used to connect to databases and process transactions.
	/// </summary>
	/// <remarks>
	/// <list type="bullet">
	/// <item><description>
	/// An instance of this class is passed as a constructor parameter
	/// all classes associated with a database table.
	/// </description></item>
	/// <item><description>
	/// This class is very useful for parameters that are needed and used throughout an application
	/// since it is passed and usually available. 
	/// </description></item>
	/// </list>
	/// </remarks>
	public class ApplicationConnection
	{
		#region Section: Processing Fields
		/// <summary>
		/// IDictionary Interface for ApplicationConfiguration
		/// </summary>
		private static IDictionary config;
		/// <summary>
		/// String used to establish a database connection.
		/// </summary>
		protected string connectString;

		/// <summary>
		/// Name of the current Windows user. 
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// This name can be used by applications for auditing-like functionality.
		/// </description></item>
		/// <item><description>
		/// It is important for the web applications that use a generic trusted user to set
		/// this value to the name of the actual user.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected string windowsUserName;

		/// <summary>
		/// Indicates whether foreign key validation for properties should be performed.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Default is 'true', perform validation checks.
		/// </description></item>
		/// <item><description>
		/// Applications can set this field to 'false' to have the foreign key checks bypass for
		/// performance reasons. However, this should be done onlt after careful consideration that
		/// the foreign keys can be trusted. 
		/// </description></item>
		/// <item><description>
		/// If this field is set to 'false', foreign key checks should be performed at the database
		/// level to insure integrity of the database.
		/// </description></item>
		/// </list>
		/// </remarks>
		private bool     doForeignKeyChecks;

		/// <summary>
		/// Indicates whether all property validations checks should be bypassed and not performed.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Default is 'false', perform validation checks.
		/// </description></item>
		/// <item><description>
		/// This field should ONLY be used for database based loads where the data is either
		/// 1) the data can be trusted to meet rules or 2) contains old data that may not meet current
		/// validity parameters but the data must be present in the database. 
		/// </description></item>
		/// <item><description>
		/// Note: Database contraint rules still may prevent data loads.
		/// </description></item>
		/// <item><description>
		/// Currently(12/2001), this property is not being used by any application.
		/// </description></item>
		/// </list>
		/// </remarks>
		private bool     bypassValidation;
		
		/// <summary>
		/// Connection to the database based on the connection string parameters.
		/// </summary>
		/// <remarks>
		/// Currently(12/2001), this field is not being used. It may have some utility if
		/// it is decided to retain a connection between database access processes.
		/// </remarks>
		private IDbConnection connection;

		/// <summary>
		/// Connection to the database based on the connection string parameters that is used by
		/// insert, update and delete transactions.
		/// </summary>
		/// <remarks>
		/// This field is necessary in order to hold a reference to a connection 
		/// during nested transactions. (A transaction is performed against a connection
		/// and all nested transactions must be from the same connection.)
		/// </remarks>
		private IDbConnection transactionConnection;

		/// <summary>
		/// Database transaction instance assoicated with the transactionConnection.
		/// </summary>
		private IDbTransaction transaction;

		///private static SqlDataAdapter da;

		///private static DataSet ds;

		private static int Success;
		/// <summary>
		/// Nested level of the transaction.
		/// </summary>
		private int transactionLevel;

		#endregion
	

		#region Section: Property Definitions

		/// <summary>
		/// String used to establish a database connection.
		/// </summary>
		public string ConnectString 
		{
			get {return connectString;}
			set {connectString = value;}
		}

		/// <summary>
		/// Name of the current Windows user. 
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>This name can be used by applications for auditing-like functionality.
		/// </description></item>
		/// <item><description>
		/// It is important for the web applications that use a generic trusted user to set
		/// this value to the name of the actual user.
		/// </description></item>
		/// </list>
		/// </remarks>
		public string WindowsUserName 
		{
			get {return windowsUserName;}
			set {windowsUserName = value;}
		}

		// TODO: Code Enhancement Possibility: (1) Add more swithes like 'DoForeignKeyChecks' or build and array of switches.
		/// <summary>
		/// Indicates whether foreign key validation for properties should be performed.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Default is 'true', perform validation checks.
		/// </description></item>
		/// <item><description>
		/// Applications can set this property to 'false' to have the foreign key checks bypassed for
		/// performance reasons. However, this should be done only after careful consideration that
		/// the foreign keys can be trusted. 
		/// </description></item>
		/// <item><description>
		/// If this property is set to 'false', foreign key checks should be performed at the database
		/// level to insure integrity of the database.
		/// </description></item>
		/// </list>
		/// </remarks>
		public bool DoForeignKeyChecks
		{
			get {return doForeignKeyChecks;}
			set {doForeignKeyChecks = value;}
		}

		/// <summary>
		/// Indicates whether all property validations checks should be bypassed and not performed.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Default is 'false', perform validation checks.
		/// </description></item>
		/// <item><description>
		/// This property should ONLY be used for database based loads where the data is either
		/// 1) the data can be trusted to meet rules or 2) contains old data that may not meet current
		/// validity parameters but the data must be present in the database. Note: Database contraint
		/// rules still may prevent data loads.
		/// </description></item>
		/// <item><description>
		/// Currently(12/2001), this property is not being used by any application.
		/// </description></item>
		/// </list>
		/// </remarks>
		public bool BypassValidation 
		{
			get {return bypassValidation;}
			set {bypassValidation = value;}
		}

		/// <summary>
		/// Connection to the database based on the connection string parameters.
		/// </summary>
		/// <remarks>
		/// Currently(12/2001), this property is not being used. It may have some utility if
		/// it is decided retain a connection for future use.
		/// </remarks>
		[System.Xml.Serialization.XmlIgnore]
		public IDbConnection Connection 
		{
			get {return connection;}
			set {connection = value;}
		}

		/// <summary>
		/// Connection to the database based on the connection string parameters that is used by
		/// insert, update and delete transactions.
		/// </summary>
		/// <remarks>
		/// This property is necessary in order to hold a reference to a connection 
		/// during nested transactions, since a transaction is performed against a connection
		/// and all nested transactions must be from the same connection.
		/// </remarks>
		[System.Xml.Serialization.XmlIgnore]
		public IDbConnection TransactionConnection
		{
			get {return transactionConnection;}
			set {transactionConnection = value;}
		}

		/// <summary>
		/// Database transaction instance assoicated with the transactionConnection.
		/// </summary>
		[System.Xml.Serialization.XmlIgnore]
		public IDbTransaction Transaction 
		{
			get {return transaction;}
			set {transaction = value;}
		}

		/// <summary>
		/// Nested level of the transaction.
		/// </summary>
		public int TransactionLevel 
		{
			get {return transactionLevel;}
			set {transactionLevel = value;}
		}

		#endregion


		#region Section: Constructors

		/// <summary>
		/// Main construtor.
		/// </summary>
		public ApplicationConnection()
		{
			Initialize();
		}
		
		/// <summary>
		/// Construtor that accepts a connection string to be associated with the instance.
		/// </summary>
		/// <param name="connectString">Database connect string</param>
		public ApplicationConnection(string connectString)
		{
			Initialize();
			this.connectString = connectString;
		}
		
		#endregion


		#region Section: Initializers

		/// <summary>
		/// Initializes the class fields with default values.
		/// </summary>
		/// <remarks>
		/// The default values are the following:
		/// <list type="bullet">
		/// <item><description>connectString         = ""</description></item>
		/// <item><description>windowsUserName       = WindowsIdentity.GetCurrent().Name</description></item>
		/// <item><description>doForeignKeyChecks    = true</description></item>
		/// <item><description>bypassValidation      = false</description></item>
		/// <item><description>connection            = null</description></item>
		/// <item><description>TransactionConnection = null</description></item>
		/// <item><description>transaction           = null</description></item>
		/// <item><description>transactionLevel      = 0</description></item>
		/// </list>
		/// </remarks>
		private void Initialize()
		{
			config  								= (IDictionary)ConfigurationSettings.GetConfig("VssConfiguration/DBConnection");			
			//throw new Exception(ApplicationConnection.DecryptPassword((string)config["ConnectionString"]));
			connectString           = "";

			windowsUserName         = WindowsIdentity.GetCurrent().Name;
			doForeignKeyChecks      = true;
			bypassValidation        = false;
			
			connection              = null;
			TransactionConnection   = null;
			transaction             = null;
			transactionLevel        = 0;

		}
		
		#endregion

		#region Section: Utilities

		#if msSql
		/// <summary>
		/// Creates connection using connectString and sets connection field to the connection reference.
		/// </summary>
		/// <remarks>If connection field is not null then a new connection is NOT created.</remarks>
		public void Connect()
		{
			if (connection == null)
			{
				connection = new SqlConnection(connectString);
			}
		}

		/// <summary>
		/// Static Method which returns an SqlConnection Object.
		/// </summary>
		/// <returns type="SqlConnection"></returns>
		public static SqlConnection GetSqlConnection()
		{
			return new SqlConnection(ApplicationConnection.GetConnectionString());
		}

		/// <summary>
		/// Static Method which returns an SqlConnection Object.
		/// </summary>
		/// <Params>
		/// <param name="ConnectionString" type="System.String">String Containing a ConnectionString.</param>
		/// </Params>
		/// <returns type="SqlConnection"></returns>
		public static SqlConnection GetSqlConnection(string ConnectionString)
		{
			return new SqlConnection(ConnectionString);
		}

		/// <summary>
		/// Gets a connection string from the custom section of the web.config file for this application.
		/// </summary>
		/// <returns type="System.String">String representing a connection to the database.</returns>
		public static string GetConnectionString()
		{		
			try
			{
				config = (IDictionary)ConfigurationSettings.GetConfig("VssConfiguration/DBConnection");
				return (string)config["ConnectionString"];
			}
			catch(Exception ex)
			{
				throw new Exception(ex.Message);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ConnectionString"></param>
		/// <returns></returns>
		public static string GetPassword(String ConnectionString)
		{
			try
			{
				int startPos = ConnectionString.IndexOf("pwd=", 0);
				int endPos = ConnectionString.IndexOf(";", startPos);
				int length = endPos - startPos;
				//throw new Exception("StartPos: " + startPos + "<BR>EndPos: " + endPos + "<BR>Length: " + length + "<BR>ConnectionString: " + ConnectionString.Substring(startPos, length));
				return ConnectionString.Substring(startPos + 4, length - 4);
			}
			catch (Exception ex)
			{
				throw new Exception(ex.Message);
			}

		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ConnectionString"></param>
		/// <returns></returns>
		public static string EncryptPassword(String ConnectionString)
		{
				
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="ConnectionString"></param>
		/// <returns></returns>
		public static string DecryptPassword(String ConnectionString)
		{


		}

		#region Section: ExecuteQuery
//		public static SqlDataReader ExecuteQuery(string Command)
//		{
//			SqlCommand sqlCmd = new SqlCommand();
//			sqlCmd.CommandText = select;
//			sqlCmd.Connection = new SqlConnection(ApplicationConnection.GetConnectionString());
//			sqlCmd.Connection.Open();
//			return sqlCmd.ExecuteReader();
//		}
		/// <summary>
		/// Executes a Query of which you expect a DataSet returned, i.e. a Non Insert or Update Command
		/// </summary>
		/// <Params>
		/// <param name="Sql" type="System.String">String containing the Query to execute</param>
		/// <param name="TableName" type="System.String">String containing the Table Name you'd like returned for your DataSet</param>
		/// </Params>
		/// <returns type="DataSet">A populated DataSet of the returning Query.</returns>
		 public static DataSet ExecuteQuery(string sqlCommand, string TableName)
		{
			 SqlDataAdapter da;
			 DataSet ds;

			 try
			{
				da = new SqlDataAdapter(sqlCommand, ApplicationConnection.GetConnectionString());
				ds = new DataSet();
				da.Fill(ds,TableName);

				da.Dispose();			
			}
			catch(Exception ex)
			{
				throw new Exception(ex.Message);
			}
			return ds;
		}

		/// <summary>
		/// Executes a Query of which you expect a DataSet returned, i.e. a Non Insert or Update Command
		/// This method has a custom timeout set for its Command Object to 300 seconds. This should be 
		/// used with the sprocs and queries taking longer time to execute (more than 30 sec).
		/// </summary>
		/// <Params>
		/// <param name="Sql" type="System.String">String containing the Query to execute</param>
		/// <param name="TableName" type="System.String">String containing the Table Name you'd like returned for your DataSet</param>
		/// </Params>
		/// <returns type="DataSet">A populated DataSet of the returning Query.</returns>
		public static DataSet ExecuteQueryTimeout(string sqlCommand, string TableName)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{
				da = new SqlDataAdapter(sqlCommand, ApplicationConnection.GetConnectionString());
				ds = new DataSet();
				da.SelectCommand.CommandTimeout = 300;
				da.Fill(ds,TableName);

				da.Dispose();			
			}
			catch(Exception ex)
			{
				throw new Exception(ex.Message);
			}
			return ds;
		}

		/// <summary>
		/// Executes a Query of which you expect a DataSet returned, i.e. a Non Insert or Update Command
		/// </summary>
		/// <Params>
		/// <param name="Sql" type="System.String">String containing the Query to execute</param>
		/// <param name="TableName" type="System.String">String containing the Table Name you'd like returned for your DataSet</param>
		/// <param name="RetryCount" type="Int32">An intereger value, for ExecuteQuery function to try & populate the dataset</param>
		/// </Params>
		/// <returns type="DataSet">A populated DataSet of the returning Query.</returns>
		public static DataSet ExecuteQueryRetry(string sqlCommand, string TableName, int RetryCount)
		{
			//SqlDataAdapter da;
			DataSet ds;

			Thread currTrd = Thread.CurrentThread;
			int Success = 0;
			string connString;
			try
			{
				connString = ApplicationConnection.GetConnectionString();
				SqlConnection conn = new SqlConnection(connString);
				SqlDataAdapter da = new SqlDataAdapter();

				// Create the SelectCommand.
				da.SelectCommand = new SqlCommand(sqlCommand, conn);

				ds = new DataSet();
				Success = da.Fill(ds,TableName);
				if (ds.Tables.Count == 0)
				{
					for(int attemptCount = 0; attemptCount < RetryCount-1; attemptCount++)
					{
						Thread.Sleep(1000);
						ds.Clear();
						da.Fill(ds,TableName);

						if (ds.Tables.Count > 0)
						{
							break;
						}
					}
				}
				da.Dispose();
				conn.Dispose();
			}
			catch(Exception ex)
			{
				throw new Exception(ex.Message) ;
			}
			return ds;
		}

		/// <summary>
		/// Executes a Query of which you expect a NameObjectCollection returned, i.e. a Non Insert or Update Command
		/// </summary>
		/// <Params>
		/// <param name="Sql" type="System.String">String containing the Query to execute</param>
		/// <param name="RetryCount" type="Int32">An intereger value, for ExecuteQuery function to try & populate the dataset</param>
		/// </Params>
		/// <returns type="DataSet">A NameObjectCollection obj of the returning Query.</returns>
		public static NameObjectCollection ExecuteReader(SqlCommand cmd)
		{
			NameObjectCollection retList = new NameObjectCollection();

			object ObjValue = new Object();
			int columnCount = 0;
			int rowCount = 0;
			string connString;
			connString = ApplicationConnection.GetConnectionString();
			SqlConnection conn = new SqlConnection(connString);
			try
			{
				SqlDataReader redr;
				conn = new SqlConnection(connString);
				conn.Open();
				cmd.Connection = conn;
				redr = cmd.ExecuteReader();

				if (redr == null)
				{
					throw new Exception("Unable to populate reader obj");
				}
				cmd.Dispose();
				
				while (redr.Read())
				{
					NameObjectCollection record = new NameObjectCollection();
					while (true)
					{
						try
						{
							ObjValue=redr[columnCount];
							columnCount++;
							record.Add("Column" + columnCount.ToString(), ObjValue);
						}
						catch (Exception)
						{
							break;
						}
					}
					columnCount = 0;
					rowCount++;
					retList.Add(rowCount.ToString(), record);
				}
				redr.Close();
				conn.Dispose();
				return retList;
			}
			catch (Exception ex)
			{
				if (conn.State == ConnectionState.Open)
					conn.Close();
				throw new Exception("Unable to populate reader obj or server is too busy.", ex);
			}
		}

		protected static void FillError(object sender, FillErrorEventArgs args) 
		{ 
			if (args.Errors.GetType() == typeof(System.OverflowException)) 
			{ 
				//Code to handle Precision Loss 
				args.Continue = true; 
			} 
			else
			{
				throw new Exception(args.Errors.Message + " FillError - (1008.0)");
			}
		} 

		/// <summary>
		/// Executes a Query of which you expect a DataSet returned, i.e. a Non Insert or Update Command
		/// </summary>
		/// <Params>
		/// <param name="SqlCommand" type="System.String">String containing the Query to execute</param>
		/// <param name="TableName" type="System.String">String containing the Table Name you'd like returned for your DataSet</param>
		/// <param name="Success" type="System.Int">Execution Success Indicator</param>
		/// </Params>
		/// <returns type="DataSet">A populated DataSet of the returning Query.</returns>
		public static DataSet ExecuteQuery(string sqlCommand, string TableName, out int Success)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{
				da = new SqlDataAdapter(sqlCommand, ApplicationConnection.GetConnectionString());
				ds = new DataSet();
				Success = da.Fill(ds,TableName);
				da.Dispose();			
			}
			catch(Exception ex)
			{
				throw ex;
			}
			return ds;
		}

		/// <summary>
		/// Executes the SqlCommand passed to the Method.
		/// </summary>
		/// <param name="sqlCommand" type="SqlCommand"/>
		/// <param name="TableName" type="System.String"/>
		/// <param name="Success" type="System.Int"/>
		/// <returns type="DataSet"/>
		
		public static DataSet ExecuteQuery(SqlCommand sqlCommand, string TableName, out int Success)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{				
				sqlCommand.Connection = new SqlConnection(ApplicationConnection.GetConnectionString());

				da = new SqlDataAdapter(sqlCommand);
				ds = new DataSet();
				Success = da.Fill(ds,TableName);
				da.Dispose();
			}
			catch(SqlException SqlEx)
			{
				throw SqlEx;
			}
			return ds;
		}

		/// <summary>
		/// Executes the SqlCommand passed to the Method.
		/// </summary>
		/// <param name="sqlCommand" type="SqlCommand"/>
		/// <param name="TableName" type="System.String"/>
		/// <returns type="DataSet"/>
		public static DataSet ExecuteQuery(SqlCommand sqlCommand, string TableName)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{
				sqlCommand.Connection = new SqlConnection(ApplicationConnection.GetConnectionString());

				da = new SqlDataAdapter(sqlCommand);
				ds = new DataSet();
				Success = da.Fill(ds,TableName);
				da.Dispose();
			}
			catch(SqlException SqlEx)
			{
				throw SqlEx;
			}
			return ds;
		}

		#endregion

		#region Section: ExecuteNonQuery
		/// <summary>
		/// Executes a Query of which you expect a DataSet returned, i.e. a Non Insert or Update Command
		/// </summary>
		/// <Params>
		/// <param name="SqlCommand" type="System.String">String containing the Query to execute</param>
		/// </Params>
		/// <returns type="DataSet">A populated DataSet of the returning Query.</returns>
		public static int ExecuteNonQuery(string sqlCommand)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{
				da = new SqlDataAdapter(sqlCommand, ApplicationConnection.GetConnectionString());
				ds = new DataSet();
				Success = da.Fill(ds);
				da.Dispose();			
			}
			catch(SqlException SqlEx)
			{
				throw SqlEx;
			}
			return Success;
		}
		
		public static int ExecuteNonQuery(string StoredProc, SqlParameter sqlParam)
		{
			try
			{
				SqlCommand sqlCommand = new SqlCommand(StoredProc);
				sqlCommand.Connection = new SqlConnection(ApplicationConnection.GetConnectionString());
				sqlCommand.Parameters.Add(sqlParam);
				return sqlCommand.ExecuteNonQuery();
			}
			catch(SqlException SqlEx)
			{
				throw SqlEx;
			}
		}

		public static int ExecuteNonQuery(string StoredProc, SqlParameterCollection sqlParams)
		{
			try
			{
				SqlCommand sqlCommand = new SqlCommand(StoredProc);
				sqlCommand.Connection = new SqlConnection(ApplicationConnection.GetConnectionString());
				foreach(SqlParameter sqlParam in sqlParams)
				{
					sqlCommand.Parameters.Add(sqlParam);
				}
				return sqlCommand.ExecuteNonQuery();
			}
			catch(SqlException SqlEx)
			{
				throw SqlEx;
			}
		}
		#endregion
		#region Section: ReturnXmlSchema
		/// <summary>
		/// Executes a Query of which you expect an XmlSchema returned.
		/// </summary>
		/// <Params>
		/// <param name="SqlCommand" type="System.String">String containing the Query to execute</param>
		/// </Params>
		/// <returns type="System.String">Returns an XmlSchema String of the DataSet</returns>
		public static string ReturnXmlSchema(string SqlCommand)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{
				da = new SqlDataAdapter(SqlCommand, ApplicationConnection.GetConnectionString());
				ds = new DataSet();
				da.Fill(ds);
				da.Dispose();			
			}
			catch(Exception ex)
			{
				throw ex;
			}
			return ds.GetXmlSchema();
		}

		/// <summary>
		/// Executes a Query of which you expect an Xml String returned.
		/// </summary>
		/// <Params>
		/// <param name="SqlCommand" type="System.String">String containing the Query to execute</param>
		/// </Params>
		/// <returns type="System.String">Returns a System.String with the Xml of the DataSet.</returns>
		public static string ReturnXmlData(string SqlCommand)
		{
			SqlDataAdapter da;
			DataSet ds;

			try
			{
				da = new SqlDataAdapter(SqlCommand, ApplicationConnection.GetConnectionString());
				ds = new DataSet();
				da.Fill(ds);
				da.Dispose();			
			}
			catch(Exception ex)
			{
				throw ex;
			}
			return ds.GetXml();
		}
		#endregion
		#region Section: Paging
		/// <summary>
		/// Pages through a dataset given the properties in the SelectCriteria and the given TableName. 
		/// </summary>
		/// <param name="dsCommand" type="DataSet"/>
		/// <param name="TableName" type="String"/>
		/// <param name="sc" type="SelectCriteria"/>
		/// <returns type="DataSet"/>
		private static DataSet PageDataSet(DataSet dsCommand, string TableName, SelectCriteria sc)
		{			
			DataTable dt = new DataTable(sc.TableName);
			
			sc.Count = 0;
			sc.AtEnd = true;
			
			for(int i = sc.StartAt; i < (sc.EndAt + 1); i++)
			{
				if (sc.Count == sc.MaxRecords) 
				{
					sc.AtEnd = false;
					break;
				}
				++sc.Count;

				dt.Rows.Add(dsCommand.Tables[TableName].Rows[i]);
			}

			DataSet ds = new DataSet();
			ds.Tables.Add(dt);			

			return ds;
		}
		/// <summary>
		/// Pages through a dataset given the properties in the SelectCriteria. 
		/// </summary>
		/// <param name="dsCommand" type="DataSet"/>
		/// <param name="TableName" type="String"/>
		/// <param name="sc" type="SelectCriteria"/>
		/// <returns type="DataSet"/>
		private static DataSet PageDataSet(DataSet dsCommand, SelectCriteria sc)
		{			
			DataTable dt = new DataTable(sc.TableName);
			
			sc.Count = 0;
			sc.AtEnd = true;
			
			for(int i = sc.StartAt; i < (sc.EndAt + 1); i++)
			{
				if (sc.Count == sc.MaxRecords) 
				{
					sc.AtEnd = false;
					break;
				}
				++sc.Count;

				dt.Rows.Add(dsCommand.Tables[0].Rows[i]);
			}

			DataSet ds = new DataSet();
			ds.Tables.Add(dt);			

			return ds;
		}
		/// <summary>
		/// Pages through a dataset given the properties in the SelectCriteria.
		/// </summary>
		/// <param name="Command" type="String"/>
		/// <param name="sc" type="SelectCriteria"/>
		/// <returns type="DataSet"/>
		private static DataSet PageDataSet(string Command, SelectCriteria sc)
		{
			DataTable dt = new DataTable(sc.TableName);
			
			sc.Count = 0;
			sc.AtEnd = true;

			DataSet dsCommand = ApplicationConnection.ExecuteQuery(Command, "QueryResult");
			
			for(int i = sc.StartAt; i < (sc.EndAt + 1); i++)
			{
				if (sc.Count == sc.MaxRecords) 
				{
					sc.AtEnd = false;
					break;
				}
				++sc.Count;

				dt.Rows.Add(dsCommand.Tables[0].Rows[i]);
			}

			DataSet ds = new DataSet();
			ds.Tables.Add(dt);			

			return ds;
		}
		#endregion
		/// <summary>
		/// Executes an SP which defines its own XML Structure and doesn't rely on SQL Server to parse it.
		/// </summary>
		/// <returns type="XmlDocument">Returns a System.Xml.XmlDocument.</returns>
		public static XmlDocument GetXmlDivisionList(String Auth_User)
		{
			XmlDocument XmlDoc;
			SqlCommand forXmlCommand;
			SqlConnection sqlConnection;
			using (sqlConnection = new SqlConnection(ApplicationConnection.GetConnectionString()))
			{
				try
				{
					forXmlCommand = new SqlCommand("VtkDivisionList");
					forXmlCommand.CommandType = CommandType.StoredProcedure;
						
					SqlParameter ParamNtUser = new SqlParameter("@NtUserName", SqlDbType.VarChar);
					ParamNtUser.Direction = ParameterDirection.Input;
					ParamNtUser.Value = Auth_User;
					forXmlCommand.Parameters.Add(ParamNtUser);

					sqlConnection.Open();

					forXmlCommand.Connection = sqlConnection;
					
					XmlDoc = new XmlDocument();
					XmlDoc.Load(forXmlCommand.ExecuteXmlReader());
					sqlConnection.Close();			
				}
				catch(Exception ex)
				{
					throw ex;
				}
				return XmlDoc;
			}
		}
		#endif
		
		#endregion
	}
}
