#define msSql

#region: Section Using

using System;
using System.Collections;
using System.Data;
using System.Data.SqlClient;
using si = Vha.Vtk.Kernel.Common.ReturnStatusIndicator;

#endregion

namespace Vha.Vtk.Kernel.Common
{

	/// <summary>
	/// Abstract class inherited by table class wrappers.
	/// Contains the fields, properties and methods required by all table based classes.
	/// </summary>
	public abstract class EntityBase: IDisposable
	{
		#region Section: Processiong Fields

		/// <summary>
		/// Indicates if instance has changed.
		/// </summary>
		protected bool isChanged;

		/// <summary>
		/// Indicates if instance is new (i.e. no database entry exists).
		/// </summary>
		protected bool isNew;

		/// <summary>
		/// Indicates that instance is marked for deletion.
		/// </summary>
		protected bool isDeleted;

		/// <summary>
		/// Indicates whether or not an instance is allowed to be deleted.
		/// </summary>
		protected bool allowDelete;

		/// <summary>
		/// Indicates whether or not an instance marked for deletion can be unmarked for deletion.
		/// </summary>
		protected bool allowUnDelete;

		/// <summary>
		/// Reference to a return status instance that can be used during the processing of the instance.
		/// </summary>
		protected ReturnStatus returnStatus;

		/// <summary>
		/// Reference to an ApplicationConnection instance to be used by the instance for database connectivity and
		/// transaction processing.
		/// </summary>
		protected ApplicationConnection appCon;

		/// <summary>
		/// Reference to an IDataSrv instance that is used as data acccess layer for this instance.
		/// </summary>
		protected IDataSrv dataSrv;
		
		#endregion


		#region Section: Property Definitions

		/// <summary>
		/// Indicates if instance has changed.
		/// </summary>
		public bool IsChanged 
		{
			get {return isChanged;}
		}

		/// <summary>
		/// Indicates if instance is new (i.e. no database entry exists).
		/// </summary>
		public bool IsNew 
		{
			get {return isNew;}
		}

		/// <summary>
		/// Indicates that instance is marked for deletion.
		/// </summary>
		public bool IsDeleted 
		{
			get {return isDeleted;}
		}

		/// <summary>
		/// Reference to an ApplicationConnection instance to be used by the instance for database connectivity and
		/// transaction processing.
		/// </summary>
		public ApplicationConnection AppCon
		{
			get {return appCon;}
			set {appCon = value;}
		}

		#endregion

		
		#region Section: Constructors

		/// <summary>
		/// Main constructor.
		/// </summary>
		/// <param name="appCon" type="ApplicationConnection"></param>
		public EntityBase(ApplicationConnection appCon)
		{	
			this.appCon  = appCon;
			Initialize();
		}

		/// <summary>
		/// Custom constructor.
		/// </summary>
		public EntityBase()
		{	
			this.appCon  = new ApplicationConnection();
			Initialize();
		}
		/// <summary>
		/// Custom constructor.
		/// </summary>
		/// <param name="ConnectionString" type="System.String">A Connection String for this instance of the ApplicationConnection</param>
		public EntityBase(string ConnectionString)
		{	
			this.appCon  = new ApplicationConnection(ConnectionString);
			Initialize();
		}
		#endregion


		#region Section: Initializers

		/// <summary>
		/// Initializes basic data fields for instance.
		/// </summary>
		/// <remarks>
		/// The deafaults are the following:
		/// <list type="bullet">
		/// returnStatus   = new ReturnStatus instance
		/// <item><description>
		/// isChanged      = false
		/// </description></item>
		/// <item><description>
		/// isNew          = false
		/// </description></item>
		/// <item><description>
		/// isDeleted      = false
		/// </description></item>
		/// <item><description>
		/// allowDelete    = false
		/// </description></item>
		/// <item><description>
		/// allowUnDelete  = false
		/// </description></item>
		/// <item><description>
		/// appCon.Connection = SqlConnection
		/// </description></item>
		/// <item><description>
		/// appCon.TransactionConnection = SqlConnection
		/// </description></item>
		/// </list>
		/// </remarks>
		protected virtual void Initialize()
		{
			returnStatus   = new ReturnStatus();
			isChanged      = false;
			isNew          = false;
			isDeleted      = false;
			allowDelete    = false;
			allowUnDelete  = false;

			#if msSql
			if (appCon.Connection == null)
			{
				appCon.Connection = new SqlConnection(appCon.ConnectString);
			}
			if (appCon.TransactionConnection == null)
			{
				appCon.TransactionConnection = new SqlConnection(appCon.ConnectString);
			}
			#endif
		}

		/// <summary>
		/// Retrieves DataSet for instance based on primary key and associates DataSet with instance.
		/// </summary>
		protected virtual void GetIdData()
		{
			dataSrv.GetIdData();
		}

		/// <summary>
		/// Populates instance properties with DataSet data.
		/// </summary>
		protected virtual void Populate()
		{
			dataSrv.Populate();
		}

		/// <summary>
		/// Populates instance with data from a row produced by a 'SELECT *' statement.
		/// </summary>
		/// <param name="dr">Data row containing data for instance</param>
		protected virtual void Populate(DataRow dr)
		{
			dataSrv.Populate(dr);
		}

		/// <summary>
		/// Refresh instance data by retrieving data from database
		/// </summary>
		public virtual void Refresh() 
		{
			dataSrv.GetIdData();
			dataSrv.Populate();
		}

		#endregion
		
		
		#region Section: Finalizers
		
		/// <summary>
		/// Releases the resources used by an instance.
		/// </summary>
		public abstract void Dispose();
		
		#endregion


		#region Section: Filers

		/// <summary>
		/// Default save logic that prohibits saving of records.
		/// </summary>
		/// <remarks>
		/// Classes that inherit from this class must implement a Save method in order to actually
		/// save records to the database.
		/// </remarks>
		/// <returns><see cref="ReturnStatus"/></returns>
		public virtual ReturnStatus Save()
		{
			returnStatus.Clear();
			returnStatus.Status = ReturnStatusIndicator.Fail;
			returnStatus.ErrorText = "Not allowed to 'Save' objects for class.";
			return returnStatus;
		}

		// TODO: Code Enhancement Possibility: (2) When implemented, need to investigate adding 'Events' to class definition so others can participate in 'Trigger'/
		/// <summary>
		/// Not Implememt. May want to implement using an event.
		/// </summary>
		/// <param name="action">Insert, update and delete</param>
		protected virtual void Trigger(string action)
		{
			// default of nothing for read only classes
		}

		// TODO: Code Enhancement Possibility: (2) When implmented, need to investigate adding an 'Event' to class definition so others can participate in 'PostSave'/
		/// <summary>
		/// Not Implememt. May want to implement using an event.
		/// </summary>
		protected virtual void PostSave()
		{
			// default of nothing for read only classes
		}

		/// <summary>
		/// Default validate method the always returns a successful validation of a instance.
		/// </summary>
		/// <returns><see cref="ReturnStatus"/></returns>
		public virtual ReturnStatus Validate()
		{
			returnStatus.Clear();
			return returnStatus;
		}

		/// <summary>
		/// Default validate method the always returns a successful validation of a instance.
		/// </summary>
		/// <returns><see cref="ReturnStatus"/></returns>
		public virtual ReturnStatus ValidateNew()
		{
			returnStatus.Clear();
			return returnStatus;
		}

		/// <summary>
		/// Marks instance as deleted if the AllowDelete property is set to 'true'.
		/// </summary>
		/// <returns><see cref="ReturnStatus"/></returns>
		public virtual ReturnStatus Delete()
		{
			returnStatus.Clear();
			if (!allowDelete)
			{
				returnStatus.Status = ReturnStatusIndicator.Fail;
				returnStatus.ErrorText = "Not allowed to 'Delete' objects for class.";
			}
			else
			{
				isDeleted = true;
			}
			return returnStatus;
		}

		/// <summary>
		/// Unmarks instance as deleted if the AllowUnDelete property is set to 'true'.
		/// </summary>
		/// <returns><see cref="ReturnStatus"/></returns>
		public virtual ReturnStatus UnDelete()
		{
			returnStatus.Clear();
			if (!allowUnDelete)
			{
				returnStatus.Status = ReturnStatusIndicator.Fail;
				returnStatus.ErrorText = "Not allowed to 'UnDelete' objects for class.";
			}
			else
			{
				if (isDeleted == false)
				{
					returnStatus.Status = ReturnStatusIndicator.Fail;
					returnStatus.ErrorText = "Instance is not marked deleted.";
				}
				else
				{
					isDeleted = false;		
				}
			}
			return returnStatus;
		}

		#endregion


		#region Section: Utilities

		/// <summary>
		/// Determines if any of the instances in an array of EntityBase objects are active.
		/// </summary>
		/// <param name="instanceArray">Array of objects</param>
		/// <returns>Returns 'true' is any of the instances are not marked deleted.</returns>
		public static bool IsListActive(ArrayList instanceArray)
		{
			foreach(EntityBase eb in instanceArray)
			{
				if (!eb.IsDeleted) return true;  
			}
			return false;
		}

		#endregion
	
	}
}
