// compiler directive that will cause transaction processing to be used
#define transaction

#region Section: Using Declartions

using System;
using System.Collections;
using System.Data;
using System.Text;
using System.Data.SqlClient;
using System.Security.Principal;
#endregion

namespace Vha.Vtk.Kernel.Common
{
	/// <summary>
	/// Abstract class inherited by classes that supply wrappers for MS SQL Server tables.
	/// This class contains fields, properties and methods required by all MS SQL Server based data access layer classes.
	/// </summary>
	public abstract class MsSqlDataSrv : IDisposable
	{
		#region Section: Connection Fields

		/// <summary>
		/// <see cref="ApplicationConnection"/> associated with an instance.
		/// </summary>
		protected ApplicationConnection appCon;

		/// <summary>
		/// Name of table in the database.
		/// </summary>
		protected string databaseTableName;

		/// <summary>
		/// DataSet to assoicate with instance record
		/// </summary>
		protected DataSet dsRec;

		/// <summary>
		/// Table name within dsRec to assoicate with instance
		/// </summary>
		protected string dsRecTableName;

		/// <summary>
		/// Row number within dsRecTableName to assoicate with instance
		/// </summary>
		protected int dsRecTableRow;

		#endregion

		#region Section: Finalizers
		/// <summary>
		/// Releases the resources used by an instance.
		/// </summary>
		public abstract void Dispose();

		/// <summary>
		/// Dispose of DataSet instance.
		/// </summary>
		/// <remarks>Called by Dispose method</remarks>
		protected void Clean()
		{
			if (dsRec != null) dsRec.Dispose();
		}

		#endregion

		#region Section: Transaction Processing

		/// <summary>
		/// Begins database transaction.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Opens the <see cref="ApplicationConnection.TransactionConnection" /> associated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.Transaction" /> property by calling the BeginTransaction method for the transaction connection.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 1 or is incremented by 1 for nested transactions.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected void DoBeginTransaction()
		{
			MsSqlTransaction.DoBeginTransaction(appCon);
		}
		
		/// <summary>
		/// Commits a database transaction.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Calls the Commit method of the <see cref="ApplicationConnection.Transaction" /> connection assoicated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 0 or is decremented by 1 for nested transactions.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected void DoCommitTransaction()
		{
			MsSqlTransaction.DoCommitTransaction(appCon);
		}
		
		/// <summary>
		/// Rollsback a database transaction.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Calls the Rollback method of the <see cref="ApplicationConnection.Transaction" /> connection assoicated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 0.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected void DoRollbackTransaction()
		{
			MsSqlTransaction.DoRollbackTransaction(appCon);
		}

		#endregion

		#region Section: Filers

		#endregion

		#region Section: Utilities

		/// <summary>
		/// Determines if an entry exists in a database table.
		/// </summary>
		/// <param name="idValue">Primary key to an entry in a database table</param>
		/// <param name="tableName">Database table name</param>
		/// <returns>Returns a boolean 'true' if entry exists. Otherwise, 'false' is retruned.</returns>
		public static bool DoesExist(int idValue, string tableName)
		{
			string columnName = "id";
			return DoesExist(idValue, tableName, columnName);
		}

		/// <summary>
		/// Determines if an entry exists in a database table.
		/// </summary>
		/// <param name="idValue">Primary key to an entry in a database table</param>
		/// <param name="tableName">Database table name</param>
		/// <param name="columnName">Column name to be used to test existence. This is usually the 'Id' field.</param>
		/// <returns>Returns a boolean 'true' if entry exists. Otherwise, 'false' is retruned.</returns>
		public static bool DoesExist(int idValue, string tableName, string columnName)
		{
			bool exists = false;
			string sCommand = "SELECT " + columnName + 
				" FROM  " + tableName +
				" WHERE " + columnName + " = '" + idValue.ToString() + "' ";
			
			SqlDataAdapter da = new SqlDataAdapter(sCommand, ApplicationConnection.GetConnectionString());
			DataSet ds = new DataSet();
			da.Fill(ds,"DoesExists");
			exists = (ds.Tables["DoesExists"].Rows.Count > 0);
			da.Dispose();
			ds.Dispose();
			return exists;
		}

		/* Added 11/1/2004 MSG... allow for hours table change...
		 table is very large and will be broken up into fiscal 
		 year tables and a partitioned view will be used to 
		 reference data.
		 Partitioned views do not allow for IDENTITY columns, 
		 so ID column must go.  Use data centric primary key instead.*/

		/// <summary>
		/// Determines if an entry exists in a database table.
		/// </summary>
		/// <param name="iVolunteerFK">Primary key value element for VtkVolunteerHours entry</param>
		/// <param name="iStationFK">Primary key value element for VtkVolunteerHours entry</param>
		/// <param name="iOrganizationFK">Primary key value element for VtkVolunteerHours entry</param>
		/// <param name="iScheduleFK">Primary key value element for VtkVolunteerHours entry</param>
		///  <param name="iBenefitingServiceFK">Primary key value element for VtkVolunteerHours entry</param>
		/// <param name="dDateWorked">Primary key value element for VtkVolunteerHours entry</param>
		/// <returns>Returns a boolean 'true' if entry exists. Otherwise, 'false' is retruned.</returns>
		public static bool DoesExist(int iVolunteerFK,
									 int iStationFK,
									 int iOrganizationFK,
									 int iScheduleFK,
									 int iBenefitingServiceFK,
									 DateTime dDateWorked)
		{
			bool exists = false;
			string sCommand = "EXEC sp_VolunterHoursDoesExist " +
				iVolunteerFK.ToString() + ", " +
				iStationFK.ToString().ToString() + ", " +
				iOrganizationFK.ToString() + ", " +
				iScheduleFK.ToString() + ", " +
				iBenefitingServiceFK.ToString() + ", " +
				dDateWorked.ToString();
			
			SqlDataAdapter da = new SqlDataAdapter(sCommand, ApplicationConnection.GetConnectionString());
			DataSet ds = new DataSet();
			da.Fill(ds,"DoesExists");
			exists = (ds.Tables["DoesExists"].Rows.Count > 0);
			da.Dispose();
			ds.Dispose();
			return exists;
		}
		#endregion
	}

	/// <summary>
	/// Abstract class implmented by collection classes that contains an array of MS SQL Server warpper class instances
	/// </summary>
	public abstract class MsSqlDataSrvCollection : CollectionBase 
	{
        
		/// <summary>
		/// <see cref="ApplicationConnection"/> associated with an instance.
		/// </summary>
		protected ApplicationConnection appCon;
        

		#region Section: Transaction Processing
		
		/// <summary>
		/// Begins database transaction.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Opens the <see cref="ApplicationConnection.TransactionConnection" /> associated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.Transaction" /> property by calling the BeginTransaction method for the transaction connection.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 1 or is incremented by 1 for nested transactions.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected void DoBeginTransaction()
		{
			MsSqlTransaction.DoBeginTransaction(appCon);
		}
		
		/// <summary>
		/// Commits a database transaction.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Calls the Commit method of the <see cref="ApplicationConnection.Transaction" /> connection assoicated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 0 or is decremented by 1 for nested transactions.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected void DoCommitTransaction()
		{
			MsSqlTransaction.DoCommitTransaction(appCon);
		}
		
		/// <summary>
		/// Rollsback a database transaction.
		/// </summary>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Calls the Rollback method of the <see cref="ApplicationConnection.Transaction" /> connection assoicated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 0.
		/// </description></item>
		/// </list>
		/// </remarks>
		protected void DoRollbackTransaction()
		{
			MsSqlTransaction.DoRollbackTransaction(appCon);
		}
        
		#endregion
        
	}

	/// <summary>
	/// Transaction processing class for MS SQL Server tables wrapper classes
	/// </summary>
	public class MsSqlTransaction
	{
		/// <summary>
		/// Begins database transaction.
		/// </summary>
		/// <param name="appCon"><see cref="ApplicationConnection"/></param>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Opens the <see cref="ApplicationConnection.TransactionConnection" /> associated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.Transaction" /> property by calling the BeginTransaction method for the transaction connection.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 1 or is incremented by 1 for nested transactions.
		/// </description></item>
		/// </list>
		/// </remarks>
		internal static void DoBeginTransaction(ApplicationConnection appCon)
		{
			if (appCon.TransactionLevel == 0)
			{
				appCon.Transaction = null;
				if (appCon.TransactionConnection.State != ConnectionState.Open)
					appCon.TransactionConnection.Open();
				#if transaction
				appCon.Transaction = appCon.TransactionConnection.BeginTransaction();
				#endif
				appCon.TransactionLevel = 1;
			}
			else
			{
				++appCon.TransactionLevel;							  	
			}	
		}
		
		/// <summary>
		/// Commits a database transaction.
		/// </summary>
		/// <param name="appCon"><see cref="ApplicationConnection"/></param>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Calls the Commit method of the <see cref="ApplicationConnection.Transaction" /> connection assoicated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 0 or is decremented by 1 for nested transactions.
		/// </description></item>
		/// </list>
		/// </remarks>
		internal static void DoCommitTransaction(ApplicationConnection appCon)
		{
			if (appCon.TransactionLevel == 1)
			{
				#if transaction
				appCon.Transaction.Commit();	
				#endif
				appCon.TransactionConnection.Close();
				appCon.TransactionLevel = 0;
			}
			else
			{
				--appCon.TransactionLevel;
			}
		}
		
		/// <summary>
		/// Rollsback a database transaction.
		/// </summary>
		/// <param name="appCon"><see cref="ApplicationConnection"/></param>
		/// <remarks>
		/// <list type="bullet">
		/// <item><description>
		/// Calls the Rollback method of the <see cref="ApplicationConnection.Transaction" /> connection assoicated with the instance.
		/// </description></item>
		/// <item><description>
		/// Sets the <see cref="ApplicationConnection.TransactionLevel" /> property of the <see cref="ApplicationConnection" /> property to 0.
		/// </description></item>
		/// </list>
		/// </remarks>
		internal static void DoRollbackTransaction(ApplicationConnection appCon)
		{
			if (appCon.TransactionLevel != 0)
			{
				#if transaction
				appCon.Transaction.Rollback();
				#endif
				appCon.TransactionConnection.Close();
				appCon.TransactionLevel = 0;
			}
		}
	
	}
	/// <summary>
	/// Utility methods to help proces MS SQL Server SQL commands 
	/// </summary>
	public class MsSqlTools
	{
		#region Section: Utilities
		/// <summary>
		/// Method that prepares a string for SQL 'WHERE' clause use. Initially,
		/// this method just replaces a single quote (') with two single quotes ('').
		/// 
		/// This is needed for statements similar to "WHERE name LIKE 'O''BRIEN%'"
		/// </summary>
		/// <param name="str">String to be prepared.</param>
		public static string PrepareString(string str)
		{
			StringBuilder x = new StringBuilder(str);
			x.Replace("'","''");
			return x.ToString();
		}

		/// <summary>
		/// This method is used to build external command syntax for a date-type field
		/// for MS SQL.
		/// </summary>
		/// <param name="fieldName">Name of date field</param>
		public static string GetExternalDateSyntax(string fieldName)
		{
			return "CONVERT(varchar, " + fieldName + ", 101)";
		}
		#endregion
	}
}
