using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using System.Web.UI.Design;
using System.IO;

[ assembly:TagPrefix("Vha.Vtk.WebControlLibrary", "VhaVtk"), 
Description("Contains ASP.NET web controls used in the PATS application.")]
namespace Vha.Vtk.WebControlLibrary
{
	/// <summary>
	///  Web developers can use this control for enhanced date handling on an ASP.NET web form. It 
	///  encapsulates three dropdown lists for month, day and year. It provides functionality to 
	///  both set the date, and read the date back after a postback.
	/// </summary>
	[DefaultProperty("Date"),
	ToolboxData("<VhaVtk:VtkCompositeDate runat=server></VhaVtk:VtkCompositeDate>"),
	Designer("Vha.Vtk.WebControlLibrary.VtkCompositeDateDesigner, Vha.Vtk.WebControlLibrary"),
	Description("Provides a compact date-handling control for ASP.NET web forms. It encapsulates three dropdown lists for month, day and year.")]
	public class VtkCompositeDate : Control, INamingContainer
	{
		#region Constants

		const string monthText = "Month";
		const string dayText = "Day";
		const string yearText = "Year";

		const string PropertyYearListBegin="VtkCompositeDateYearListBegin";
		const string PropertyYearListEnd="VtkCompositeDateYearListEnd";
		const string PropertyFirstItemHeader="VtkCompositeDateFirstItemHeader";
		const string PropertyDate="VtkCompositeDateDate";
		const string PropertyEnabled="VtkCompositeDateEnabled";

		const int MonthListPosition=0;
		const int DayListPosition=2;
		const int YearListPosition=4;

		#endregion

		#region Properties

		// private DateTime date;
		/// <summary>
		/// Set the date value of the control by setting this property to a date. If an invalid date
		/// is set into the control, the corresponding error message is returned in the 
		/// <see cref="VtkCompositeDate.ValidateErrorMessage"/> property.
		/// 
		/// DEBUGGING NOTE: When debugging the control, setting a watch on the DATE property exercises the DATE 
		/// "GET" accessor. But, the get accessor MODIFIES the current value of the date in ViewState by first 
		/// reading the date as specified in the three drop down lists, and then setting that into ViewState and 
		/// returning that value. This works well EXCEPT if you're debugging the control code itself, at a point 
		/// when the dropdown lists have only just been created, and you have a watch set on the DATE property.
		/// At this point, any date value set at designtime is auto-changed to DateTime.MinValue. This is an 
		/// unfortunate side-effect of having code that "does something" in the Date get accessor. However, the
		/// code is necessary to allow the control to support both designtime and runtime access to the date
		/// property, linked to the three dropdown list values.
		/// </summary>
		[Bindable(false), Category("Data"), ReadOnly(false),
		Description("Date to set the control to.")]
		public DateTime Date
		{
			get 
			{
				if (Controls.Count>0) 
				{
					// if drop down lists are created
					try 
					{
						// get the date value according to the dropdownlists
						ViewState[PropertyDate] = GetDateFromControls();
						ValidateErrorMessage="";
					} 
					catch (Exception err)
					{
						ViewState[PropertyDate]=DateTime.MinValue;
						ValidateErrorMessage=err.Message;
					}
				}
				if (ViewState.Count>0) 
				{
					if (this.ViewState[PropertyDate]!=null) 
					{
						return (DateTime) this.ViewState[PropertyDate];
					} 
					else 
					{
						return DateTime.MinValue;
					}
				} 
				else 
				{
					return DateTime.MinValue;
				}
			}
			set 
			{ 
				this.ViewState[PropertyDate] = value;
				if (Controls.Count>0) 
				{
					// if drop down lists are created
					SetVisibleDate(value);
				}
			} 
		}
		
		private string validateErrorMessage="";
		/// <summary>
		///  If the values in the three drop-down lists result in an invalid date (i.e., Feb. 31),
		///  this property contains the error message describing the problem. The property is set when
		///  the <see cref="VtkCompositeDate.Date"/> property is accessed.
		/// </summary>
		[Bindable(false), Category("Misc"), DefaultValue(""), ReadOnly(true),
		Description("Holds error message returned if the date in the date controls is invalid.")]
		public string ValidateErrorMessage 
		{
			get 
			{
				return validateErrorMessage;
			}
			set 
			{
				validateErrorMessage=value;
			}
		}

		/// <summary>
		/// The beginning of the year range to use for the Year drop down list. It should be set in conjunction 
		/// with setting <see cref="VtkCompositeDate.YearListEnd"/>."
		/// </summary>
		[Bindable(false), Category("Behavior"), DefaultValue(0), ReadOnly(false),
		Description("The beginning of the year range to use for the Year drop down list.")]
		public int YearListBegin 
		{
			get 
			{
				if (ViewState.Count>0) 
				{
					if (this.ViewState[PropertyYearListBegin]!=null) 
					{
						return (int) this.ViewState[PropertyYearListBegin];
					} 
					else 
					{
						return 0;
					}
				} 
				else 
				{
					return 0;
				}
			}
			set 
			{
				this.ViewState[PropertyYearListBegin]=value;
				if (Controls.Count>0) 
				{
					// effect from runtime set. If ddls are created...
					PopulateYearList();
					if (ViewState[PropertyDate]!=null) 
					{
						SetVisibleDate((DateTime)ViewState[PropertyDate]);
					}
				}
			}
		}

		/// <summary>
		/// The end of the year range to use for the Year drop down list. It should be set in conjunction with
		/// setting <see cref="VtkCompositeDate.YearListBegin"/>."
		/// </summary>
		[Bindable(false), Category("Behavior"), DefaultValue(0), ReadOnly(false),
		Description("The end of the year range to use for the Year drop down list.")]
		public int YearListEnd 
		{
			get 
			{
				if (ViewState.Count>0) 
				{
					if (this.ViewState[PropertyYearListEnd]!=null) 
					{
						return (int) this.ViewState[PropertyYearListEnd];
					} 
					else 
					{
						return 0;
					}
				} 
				else 
				{
					return 0;
				}
			}
			set 
			{
				this.ViewState[PropertyYearListEnd]=value;
				if (Controls.Count>0) 
				{
					// effect from runtime set. If ddls are created...
					PopulateYearList();
					if (ViewState[PropertyDate]!=null) 
					{
						SetVisibleDate((DateTime)ViewState[PropertyDate]);
					}
				}
			}
		}

		/// <summary>
		/// Deprecated. First Item Headers are always displayed now. This means there's always a place in the
		/// control to handle a null date (e.g., DateTime.MinValue). Otherwise, if the first item corresponds to,
		/// for example, 1/1/1979, then a null date transforms into an actual date. Setting this property therefore
		/// has no effect. 
		/// </summary>
		[Bindable(false), Category("Behavior"), DefaultValue(false),  ReadOnly(true),
		Description("Determines whether to include initial items of Day, Month and Year in the three drop down lists.")]
		public bool FirstItemHeader 
		{
			get 
			{
				return true;
				/*if (this.ViewState[PropertyFirstItemHeader]!=null) 
				{
					return (bool) this.ViewState[PropertyFirstItemHeader];
				} 
				else 
				{
					return false;
				}*/
			}
			set 
			{  
				/* this.ViewState[PropertyFirstItemHeader] = value;
				if (Controls.Count>0) 
				{
					// effect from runtime set. If ddls are created...
					SetHeader((DropDownList)Controls[MonthListPosition], monthText);
					SetHeader((DropDownList)Controls[DayListPosition], dayText);
					SetHeader((DropDownList)Controls[YearListPosition], yearText);
				} */
			} 
		}

		/// <summary>
		///  This property set the Enabled property of all dropdowns.
		///  The property is set when 
		///  the <see cref="VtkCompositeDate.Enabled"/> property is accessed.
		/// </summary>
		[Bindable(false), Category("Misc"), DefaultValue(""), ReadOnly(false),
		Description("Holds error message returned if the date in the date controls is invalid.")]
		public bool Enabled 
		{
			get 
			{
				return ((DropDownList)Controls[YearListPosition]).Enabled;
			}
			set 
			{
				((DropDownList)Controls[YearListPosition]).Enabled = value;
				((DropDownList)Controls[DayListPosition]).Enabled = value;
				((DropDownList)Controls[MonthListPosition]).Enabled = value;
			}
		}

		#endregion	

		#region Control Creation

		/// <summary>
		/// Overrides the default OnInit method. In the lifecycle of the control, this method is called
		///  _before_ ViewState is restored. The reason we override this method is to have a place to reflect 
		///  any design-time Date property setting into the dropdownlists. If a different date is contained 
		///  because this is a postback rather than an initial page invocation, the setting of the date here will 
		///  be overridden the first time the Date property is accessed.
		///  </summary>
		/// <param name="e">Required argument.</param>
		protected override void OnInit(EventArgs e) 
		{
			base.OnInit(e);
			// make sure drop down lists have been created
			this.EnsureChildControls();
			// set the date into the lists. Ensures that if the date is set at DESIGN time, 
			//  the designtime setting is reflected in the control lists. 
			if (ViewState.Count>0) 
			{
				if (ViewState[PropertyDate]!=null) 
				{
					SetVisibleDate((DateTime)ViewState[PropertyDate]);
				}
			}
		}

		/// <summary>
		/// This method is for internal use of the control only. It overrides the 
		/// <see cref="System.Web.UI.Control.CreateChildControls"/> method to create all child controls (the
		/// three drop down lists).
		/// </summary>
		protected override void CreateChildControls() 
		{
			// create the three drop down lists
			DropDownList monthList = new DropDownList();
			DropDownList dayList = new DropDownList();
			DropDownList yearList = new DropDownList();

			// add the drop down lists to the child control list, seperated by a space each
			Controls.AddAt(0,monthList);
			Controls.AddAt(1,new LiteralControl(" "));
			Controls.AddAt(2,dayList);
			Controls.AddAt(3,new LiteralControl(" "));
			Controls.AddAt(4,yearList);

			// populate each lists with months (1-12), days (1-31), and years (depends on YearListBegin and 
			//   YearListEnd properties)
			PopulateMonthList();
			PopulateDayList();
			PopulateYearList();

			// insert a first line header ("month", "day", "year")
			SetHeader((DropDownList)Controls[MonthListPosition], monthText);
			SetHeader((DropDownList)Controls[DayListPosition], dayText);
			SetHeader((DropDownList)Controls[YearListPosition], yearText);
		}

		/// <summary>
		/// Populates the year dropdownlist with year items ranging from the current value of 
		/// <see cref="VtkCompositeDate.YearListBegin"/> 
		/// to the current value of <see cref="VtkCompositeDate.YearListEnd"/>.
		/// </summary>
		private void PopulateYearList() 
		{
			DropDownList yearList = (DropDownList)Controls[YearListPosition];
			// rebuild the list from scratch each time
			yearList.Items.Clear();
			// put in the header
			SetHeader(yearList, yearText);
			// put in the years
			for (int i=this.YearListBegin;i<=this.YearListEnd;i++) 
			{
				yearList.Items.Add(new ListItem(i.ToString(),i.ToString()));
			}
		}

		/// <summary>
		/// Populates the day dropdownlist with the 31 days of the month.
		/// </summary>
		private void PopulateDayList() 
		{
			DropDownList dayList = (DropDownList)Controls[DayListPosition];
			if (dayList.Items.Count<31) 
			{
				for (int i=1;i<=31;i++) 
				{
					ListItem li = new ListItem(i.ToString(),i.ToString());
					dayList.Items.Add(li);
				}
			}
		}

		/// <summary>
		/// Populates the month dropdownlist with the items for the 12 months of the year.
		/// </summary>
		private void PopulateMonthList() 
		{
			DropDownList monthList = (DropDownList)Controls[MonthListPosition];
			if (monthList.Items.Count<12) 
			{
				for (int i=1;i<=12;i++) 
				{
					ListItem li = new ListItem(i.ToString(),i.ToString());
					switch (i) 
					{ 
						case 1: li.Text="January";
							break;
						case 2: li.Text="February";
							break;
						case 3: li.Text="March";
							break;
						case 4: li.Text="April";
							break;
						case 5: li.Text="May";
							break;
						case 6: li.Text="June";
							break;
						case 7: li.Text="July";
							break;
						case 8: li.Text="August";
							break;
						case 9: li.Text="September";
							break;
						case 10: li.Text="October";
							break;
						case 11: li.Text="November";
							break;
						case 12: li.Text="December";
							break;
						default:
							break;
					}
					monthList.Items.Add(li);
				}
			}
		}

		#endregion

		#region Date Get

		/// <summary>
		/// Gets the date from the dropdownlist controls. NOTE: This method throws an exception if the date in
		/// the controls is invalid.
		/// </summary>
		/// <returns>The date reflected by the current settings of the three dropdownlists.</returns>
		private DateTime GetDateFromControls() 
		{
			/* start new code 10/9/02 */
			// purpose is to set a different error message into the controls -- "Not selected" -- if the user
			// simply didn't select any date at all, as opposed to entering a malformed date.
			int yearPos=Convert.ToInt32(((DropDownList)Controls[YearListPosition]).SelectedItem.Value);
			int monthPos=Convert.ToInt32(((DropDownList)Controls[MonthListPosition]).SelectedItem.Value);
			int dayPos=Convert.ToInt32(((DropDownList)Controls[DayListPosition]).SelectedItem.Value);
			
			if ((yearPos==-1)&&(monthPos==-1)&&(dayPos==-1))
			{
				throw new Exception("Not selected");					
			}
			
			/* basically the end of new code */

			// note: this method throws an exception if the date in the controls is invalid
			DateTime dtDate = DateTime.MinValue;
			this.EnsureChildControls();
			// create date from current values in controls
			dtDate=new DateTime(yearPos, monthPos, dayPos);
			return dtDate;
		}

		#endregion
        
		#region Date Set

		/// <summary>
		/// Sets the visible date in the three drop down lists.
		/// </summary>
		/// <param name="dtDate">The date to set the three drop down lists to reflect.</param>
		private void SetVisibleDate(DateTime dtDate) 
		{
			if (dtDate == DateTime.MinValue) 
			{
				// TODO: add a "none" or "not set" item into the lists here to handle this "non"-date?
				((DropDownList)Controls[MonthListPosition]).SelectedIndex=0;
				((DropDownList)Controls[DayListPosition]).SelectedIndex=0;
				((DropDownList)Controls[YearListPosition]).SelectedIndex=0;
			} 
			else 
			{
				// set the month, day and year lists to reflect the date
				SetMonth(dtDate.Month);
				SetDay(dtDate.Day);
				SetYear(dtDate.Year);
			} 			
		}

		/// <summary>
		/// Inserts or removes the "FirstItemHeader" from the three dropdown lists based on the 
		/// <see cref="VtkCompositeDate.FirstItemHeader"/> property being true or false. 
		/// </summary>
		/// <param name="ddlList">The list to manage the header setting for -- any one of the three dropdown
		/// lists.</param>
		/// <param name="headerText">The text to set the first item of the list (pass whether the
		/// <see cref="VtkCompositeDate.FirstItemHeader"/> property is true or false.)</param>
		private void SetHeader(DropDownList ddlList, string headerText) 
		{
			/* // if we DO need to ensure the list has a header
			if (FirstItemHeader) 
			{*/

			// if the list has been initialized
			if (ddlList.Items.Count>0) 
			{
				// if the item at position 0 is present
				if (ddlList.Items[0]!=null) 
				{
					// if the value of the item at position 0 is NOT -1, meaning a header is NOT already present
					if (ddlList.Items[0].Value!="-1") 
					{
						// insert the header text at position 0
						ddlList.Items.Insert(0,new ListItem(headerText,"-1"));
					}
				} 
			}
			else 
			{
				// if the list has not been initialized, add header text as a first item
				ddlList.Items.Insert(0,new ListItem(headerText,"-1"));
			}

			/* } 
			else // clear first item header if needed
			{
				// if the list has been initialized
				if (ddlList.Items.Count>0) 
				{
					// if there is an item at position 0
					if (ddlList.Items[0]!=null) 
					{
						// if the value of that item is -1, it's a header
						if (ddlList.Items[0].Value=="-1") 
						{
							// remove the header at position 0
							ddlList.Items.RemoveAt(0);
						}
					} 
				}
			}*/
		}

		/// <summary>
		/// Set the selected item in the month list based on the int value of the month passed in.
		/// </summary>
		/// <param name="month">Month to set the monthlist to (1-12).</param>
		private void SetMonth(int month) 
		{
			DropDownList MonthList = ((DropDownList)Controls[MonthListPosition]);
			// loop through month list till find a matching item
			for (int i=0;i<MonthList.Items.Count;i++) 
			{
				if (MonthList.Items[i].Value==month.ToString()) 
				{
					MonthList.SelectedIndex=i;
					break;
				}
			}
		}

		/// <summary>
		/// Set the selected item in the day list based on the int value of the day passed in.
		/// </summary>
		/// <param name="day">Day to set the daylist to (1-31).</param>
		private void SetDay(int day) 
		{
			DropDownList DayList=((DropDownList)Controls[DayListPosition]);
			// loop through day list till find a matching item
			for (int i=0;i<DayList.Items.Count;i++) 
			{
				if (DayList.Items[i].Value==day.ToString()) 
				{
					DayList.SelectedIndex=i;
					break;
				}
			}
		}

		/// <summary>
		/// Set the selected item in the year list based on the int value of the year passed in. If the year
		/// passed in is outside the date range specified by YearListBegin and YearListEnd, those properties are
		/// changed to include the year passed in (which repopulates the year list, so that there is a list item
		/// present to select).
		/// </summary>
		/// <param name="year">Year to set the yearlist to.</param>
		private void SetYear(int year) 
		{
			DropDownList yearList = (DropDownList)Controls[YearListPosition];
			if (year<YearListBegin) 
			{ 
				// re-populate list if year is less than start of list
				YearListBegin=year; 
			} 
			else if (year>YearListEnd) 
			{
				// re-populate list if year is greater than end of list
				YearListEnd=year;
			} 
			else if (YearListBegin==0) 
			{
				YearListBegin=year;
			}

			// loop through yearlist till find a matching item
			for (int i=0;i<yearList.Items.Count;i++) 
			{
				if (yearList.Items[i].Value==year.ToString()) 
				{
					yearList.SelectedIndex=i;
					break;
				} 
			}
		}

		#endregion

		#region Deprecated

		/// <summary>
		///  This is a deprecated method that has been no-op'd. Existing code will not break; its functionality
		///  has been subsumed in the control itself. Formerly, this method was called after setting
		///  <see cref="VtkCompositeDate.YearListBegin"/>, 
		///  <see cref="VtkCompositeDate.YearListEnd"/>,and (optionally) 
		///  <see cref="VtkCompositeDate.FirstItemHeader"/>, and it would then set up the three dropdown lists of 
		///  the composite date control. Now, these lists are set up automatically, whether these properties are 
		///  set at designtime or runtime, with no need to call a special separate method."
		/// </summary>
		[Description("Deprecated method, that has been no-op'd. Existing code calling this method will not be affected; its functionality has been subsumed in the control itself.")]
		public void InitializeControl() 
		{
			// Deprecated method that has been no-op'd. Formerly it would ensure that all needed items were created
			//  in the dropdown lists.
			return;
		}

		#endregion
	}

	/// <summary>
	/// Used by the <see cref="Vha.Vtk.WebControlLibrary.VtkCompositeDate"/> control to display the control at design-time (in the web form
	/// designer). It shows enough of what the control will look like to make designing web forms
	/// easier.
	/// </summary>
	public class VtkCompositeDateDesigner : System.Web.UI.Design.ControlDesigner 
	{
		const string monthText = "Month";
		const string dayText = "Day";
		const string yearText = "Year";

		/// <summary>
		/// This method is called by VS.NET to display the control at design-time (in the web form
		/// designer). It shows enough of what the control will look like to make designing web forms
		/// easier.
		/// </summary>
		/// <returns>HTML string to display a facsimile of the control in the web page designer.</returns>
		public override string GetDesignTimeHtml()
		{
			StringWriter sw = new StringWriter();
			HtmlTextWriter tw = new HtmlTextWriter(sw);

			VtkCompositeDate myDate = (VtkCompositeDate) Component;

			DropDownList monthList = new DropDownList();
			DropDownList dayList = new DropDownList();
			DropDownList yearList = new DropDownList();
			LiteralControl mySpace = new LiteralControl(" ");

			monthList.Items.Clear();
			dayList.Items.Clear();
			yearList.Items.Clear();

			if (myDate.Date!=DateTime.MinValue) 
			{
				// if a design-time date is set, display the date
				monthList.Items.Add(new ListItem(myDate.Date.ToString("MMMM"),"0"));
				dayList.Items.Add(new ListItem(myDate.Date.Day.ToString(),"0"));
				yearList.Items.Add(new ListItem(myDate.Date.Year.ToString(),"0"));
			} 
			else if (myDate.FirstItemHeader) 
			{
				// if no design-time date has been set, but FirstItemHeader is true, display
				//   the first-item header strings
				monthList.Items.Add(new ListItem(monthText,"-1"));
				dayList.Items.Add(new ListItem(dayText,"-1"));
				yearList.Items.Add(new ListItem(yearText,"-1"));
			} 
			else 
			{
				// otherwise, display a date of January 1, [current year]
				monthList.Items.Add(new ListItem("January","0"));
				dayList.Items.Add(new ListItem("1","0"));
				yearList.Items.Add(new ListItem(myDate.YearListBegin.ToString(),"0"));
			}

			monthList.RenderControl(tw);
			mySpace.RenderControl(tw);
			dayList.RenderControl(tw);
			mySpace.RenderControl(tw);
			yearList.RenderControl(tw);
			return sw.ToString();
		}
	}	
}
