using System;
using System.Collections;
using System.Configuration;
using System.ComponentModel;
using System.IO;

using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.Web.SessionState;

using System.Data;
using System.Data.SqlClient;
using System.Text;
using System.Xml;
using System.Xml.XPath;

using Vha.Vss.Security;
using Vha.Vss.Utility;
using Vha.Vtk.Kernel.Common;
[ assembly:TagPrefix("Vha.Vtk.WebControlLibrary", "VhaVtk") ]
namespace Vha.Vtk.WebControlLibrary
{
	/// <summary>
	/// <para>This web control is for use by the VTK application. It should be used by ALL pages within the VTK 
	/// application where having an identified station is required. This means all pages except for those
	/// within the National directory, and also the Utility\DivSelector page. </para>
	/// <para>This control ensures
	/// that a valid-for-the-current-user station is currently selected, and provides the name and ID of that
	/// station in several properties. If a station is not currently selected, the user is redirected to the
	/// DivSelector page to choose a station.</para>
	/// <para>Note: Stations should be passed from page to page in a URL parameter. Whenever a new page is 
	/// navigated to, the current station ID should be present in the URL parameter list, in the URL 
	/// parameter returned by <see cref="VtkPageHeader.GetURLParameterString"/>().</para>
	/// <para>Note: Each time a page is loaded, the station passed in the URL Parameters is checked against a
	/// table of stations valid for the user, kept in the Session object. This prevents the user from 
	/// accessing an unauthorized station, even if the user tries to modify the URL parameters. The list is
	/// placed in the Session object at session start, by methods in VTK's global.asax.</para>
	/// </summary>

	[DefaultProperty("ChangeEnabled"),
	ToolboxData("<VhaVtk:VtkPageHeader runat=server></VhaVtk:VtkPageHeader>"),
	Designer("Vha.Vtk.WebControlLibrary.VtkPageHeaderDesigner, Vha.Vtk.WebControlLibrary")]
	public class VtkPageHeader : Control, INamingContainer
	{
		private string pageName;
		private string AppVersion = string.Empty;
		private string HelpPage = string.Empty;
		private StringBuilder ChangeDivision = null;
		private string CurrentPage = string.Empty;
		private IDictionary config;
		XmlDocument XmlDivisionList;
		XmlNodeList XmlDivisions;
		XmlDocument XmlSecurityList;

		const string monthText = "Month";
		const string dayText = "Day";
		const string yearText = "Year";
		const string URLPARAMETER="StationId";
		const string PROPERTYCHANGEENABLED="VhaPageHeaderDivChangeEnabled";
		const string PROPERTYSELECTEDNAME="VhaPageHeaderDivSelectedName";
		const string PROPERTYSELECTEDNUMBER = "VhaPageHeaderDivSelectedNumber";
		const string PROPERTYSELECTEDID = "VhaPageHeaderDivSelectedId";

		/// <summary>
		/// At designtime, set to true to enable the "change divisions" link on the control; 
		/// set to false to disable the "change divisions" link.
		/// </summary>
		[Bindable(true), Category("Behavior"), DefaultValue(true), ReadOnly(false),
		Description("Determines whether to enable or disable the link to change stations.")]
		public bool ChangeEnabled
		{
			get
			{
				if (ViewState[PROPERTYCHANGEENABLED]!=null)
				{
					return (bool) ViewState[PROPERTYCHANGEENABLED];
				}
				else 
				{
					return true;
				}
			}
			set
			{
				ViewState[PROPERTYCHANGEENABLED]=value;
			}

		}

		/// <summary>
		/// This property contains the user's currently selected Station name. It is only populated after
		/// <see cref="VtkPageHeader.IsDivSelected"/>() is called.
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue(""), ReadOnly(true), 
		Description("The user's currently selected Station Name, populated after IsDivSelected call.")]
		public string DivSelectedName 
		{
			get 
			{
				if (ViewState[PROPERTYSELECTEDNAME]!=null) 
				{
					return (string) ViewState[PROPERTYSELECTEDNAME];
				}
				else 
				{
					return "";
				}
			}
			set 
			{
				ViewState[PROPERTYSELECTEDNAME]=value;
			}
		}

		/// <summary>
		/// This property contains the user's currently selected Station number. It is only populated after
		/// <see cref="VtkPageHeader.IsDivSelected"/>() is called.
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue(""), ReadOnly(true), 
		Description("The user's currently selected Station Number, populated after IsDivSelected call.")]
		public string DivSelectedNumber
		{
			get 
			{
				if (ViewState[PROPERTYSELECTEDNUMBER]!=null) 
				{
					return (string) ViewState[PROPERTYSELECTEDNUMBER];
				} 
				else 
				{
					return "";
				}
			}
			set 
			{
				ViewState[PROPERTYSELECTEDNUMBER]=value;
			}
		}

		/// <summary>
		/// This property contains the user's currently selected Station ID. It is only populated after
		/// <see cref="VtkPageHeader.IsDivSelected"/>() is called.
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue(-1), ReadOnly(true), 
		Description("The user's currently selected Station ID, populated after IsDivSelected call.")]
		public int DivSelectedId 
		{
			get 
			{
				if (ViewState[PROPERTYSELECTEDID]!=null) 
				{
					return (int) ViewState[PROPERTYSELECTEDID];
				} 
				else 
				{
					return -1;
				}
			}
			set 
			{
				ViewState[PROPERTYSELECTEDID]=value;
			}
		}

		/// <summary>
		/// This Property contains the page the user is currently trying to access
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue(-1), ReadOnly(true), 
		Description("Contains the page the user is currently trying to access.")]
		public string PageName 
		{
			get { return pageName; }
			set { pageName=value; }
		}
		/// <summary>
		/// Returns the URL parameter in which the control always expects to contain the current station Id. 
		/// Use this method so that you know what URL parameter to place the station ID in, when building a URL
		/// to jump to (when the page being jumped to also uses the VtkPageHeader control).
		/// </summary>
		/// <returns>URL parameter key, in which the control expects to contain the current station Id.</returns>
		public static string GetURLParameterString() 
		{
			return URLPARAMETER;
		}


		/// <summary>
		/// In any page using the control, call IsDivSelected() in the Page_Load method, in the 
		/// <c>if (!IsPostBack){}</c> section. This call checks whether the user has already selected a station.
		/// If the user has previously selected a station, the values for 
		/// <see cref="VtkPageHeader.DivSelectedId"/>, 
		/// <see cref="VtkPageHeader.DivSelectedName"/>, and 
		/// <see cref="VtkPageHeader.DivSelectedNumber"/> are populated, and True is returned. If not, the user is
		/// redirected to the DivSelector page to choose a station.
		/// </summary>
		/// <returns>True if a station is selected by the user, false if no station is currently selected. Before 
		/// false is returned, however, the user is redirected to a page to select a station.</returns>
		[Description("Always call this in the Page_Load method, in the if (!IsPostBack){} section.")]
		public bool IsDivSelected() 
		{
			if(this.GetSelectedDivision())
			{
				this.EnsureChildControls();
				return true;
			}
			else
				return false;
		}
		public bool IsDivSelected(string PageName) 
		{
			this.PageName = PageName;
			return IsDivSelected();
		}
		private bool GetSelectedDivision()
		{			
			bool secure;
			try
			{
				config = (IDictionary)ConfigurationSettings.GetConfig("VssConfiguration/Application");
			}
			catch{}

			try
			{
				XmlDivisionList = new XmlDocument();
				String Auth_User = Page.Request.ServerVariables.Get("AUTH_USER");
				XmlDivisionList = ApplicationConnection.GetXmlDivisionList(Auth_User);
				Page.Trace.Warn("Authorize", XmlDivisionList.DocumentElement.SelectNodes("Division").Count.ToString());
				if(XmlDivisionList.DocumentElement.SelectNodes("Division").Count == 0) 
				{
					return false;
				}

				XmlSecurityList = new XmlDocument();
				XmlSecurityList.Load((string)config["Security"]);

				this.DivSelectedId = -1;
			
				// check if Div List in Session. If not, throw exception, return false.
				if (XmlDivisionList == null)
				{
					Page.Trace.Warn("XmlDivisionList == null", true.ToString());
					return false;
				} 
				else
				{
					if (Page.Request.QueryString["StationId"] != null) 
					{
						SetSelectedDivision(XmlDivisionList.DocumentElement.SelectSingleNode("Division[@Id=" + Page.Request.QueryString["StationId"] + "]"));
					}
					else if(XmlDivisionList.DocumentElement.Attributes.GetNamedItem("DefaultDivision") != null)
					{
						string DivisionId = XmlDivisionList.DocumentElement.Attributes.GetNamedItem("DefaultDivision").Value;
						SetSelectedDivision(XmlDivisionList.DocumentElement.SelectSingleNode("Division[@Id=" + DivisionId + "]"));						
					}
					else if(XmlDivisionList.DocumentElement.ChildNodes.Count > 0)
					{
						SetSelectedDivision(XmlDivisionList.DocumentElement.ChildNodes[0]);
					}
				}
			}
			catch
			{
				return false;
			}

			Page.Trace.Warn("PageName", PageName);
			Page.Trace.Warn("XmlDivision", XmlDivisionList.OuterXml);
			Page.Trace.Warn("DivSelectedId", DivSelectedId.ToString());

			if(PageName != "AUTHORIZE")
				secure = Security.IsInRole(PageName, XmlDivisionList, XmlSecurityList, DivSelectedId.ToString());
			else
				secure = true;

			return secure;			
		}
		private bool SetSelectedDivision(XmlNode XmlDivision) 
		{
			DivSelectedId=Convert.ToInt32(XmlDivision.Attributes.GetNamedItem("Id").Value);
			DivSelectedNumber=XmlDivision.Attributes.GetNamedItem("Number").Value;
			DivSelectedName=XmlDivision.Attributes.GetNamedItem("Name").Value;
			return true;
		}

		/// <summary>
		/// This method is for internal use of the control only. It overrides the 
		/// <see cref="System.Web.UI.Control.CreateChildControls"/> method.
		/// </summary>
		protected override void CreateChildControls() 
		{
			try
			{
				config = (IDictionary)ConfigurationSettings.GetConfig("VssConfiguration/Application");
			}
			catch{}
			//Create the Division Label			
			ChangeDivision = new StringBuilder();
			string SelectedDivision = DivSelectedId.ToString();
			
			if(XmlDivisionList == null || XmlSecurityList == null)
			{
				this.GetSelectedDivision();
			}
			XmlDivisions = XmlDivisionList.DocumentElement.ChildNodes;
			
			if(!this.ChangeEnabled) 
				ChangeDivision.Append(XmlDivisionList.DocumentElement.SelectSingleNode("Division[@Id='" + SelectedDivision + "']/@DisplayName").Value);
			else
			{
				ChangeDivision.Append("<select name=\"ChangeDivision\" Id=\"ChangeDivision\" OnChange=\"javascript:ChangeDivisions('" + Page.Request.Url.AbsolutePath + "?StationId=', this)\">");

				foreach(XmlNode XmlDiv in XmlDivisions)
				{
					string DisplayName = XmlDiv.Attributes.GetNamedItem("DisplayName").Value;
					string Id = XmlDiv.Attributes.GetNamedItem("Id").Value;
				
					if(SelectedDivision != Id)
						ChangeDivision.Append("<option value=\"" + Id + "\">" + DisplayName + "</option>");
					else					
					{
						ChangeDivision.Append("<option value=\"" + Id + "\" selected>" + DisplayName + "</option>");					
					}
				}
			}
			ChangeDivision.Append("</select>");
			
			Page.RegisterClientScriptBlock("ChangeDivision", "<script language=\"javascript\" src=\"/VTK/Utility/ChangeDivisions.js\"></Script>");
			Page.RegisterClientScriptBlock("Help", "<script language=\"javascript\" src=\"/VTK/Utility/OpenWindow.js\"></script>");
			AppVersion = (string)config["Version"];

			CurrentPage = Page.Request.Url.Segments.GetValue(Page.Request.Url.Segments.GetLength(0)-1).ToString();
			HelpPage = CurrentPage.Replace(".aspx", "_Help.htm");


			//Add the whole Table to the Controls Object
			Controls.Add(CreatePageHeaderTable());
		} 

		public bool IsInVtkRole(string roleName, int stationId)
		{
			if(XmlDivisionList.DocumentElement.SelectSingleNode("Division[@Id=" + stationId.ToString() + "]/Role[@Name='" + roleName + "']") != null)
				return true;
			else
				return false;
		}

		private Table CreatePageHeaderTable()
		{
			Table _Table = new Table();
			_Table.CellPadding = 0;
			_Table.CellSpacing = 0;
			_Table.Attributes.Add("Border", "0");
			_Table.BackColor = System.Drawing.ColorTranslator.FromHtml("#eeeeee");
			_Table.Width = Unit.Percentage(100);

			_Table.Rows.Add(new TableRow());
			TableRow _Row = _Table.Rows[0];

			//Add the Station Selection Cell
			_Row.Cells.Add(new TableCell());
			TableCell _StationCell = _Row.Cells[0];
			_StationCell.Attributes.Add("align", "left");
			_StationCell.Attributes.Add("width", "33%");
			_StationCell.Controls.Add(new LiteralControl("Station: " + ChangeDivision.ToString()));

			//Add the Version Cell
			_Row.Cells.Add(new TableCell());
			TableCell _VersionCell = _Row.Cells[1];
			_VersionCell.Attributes.Add("align", "center");
			_VersionCell.Attributes.Add("width", "33%");
			_Table.Rows[0].Cells[1].Controls.Add(new LiteralControl("<B>V" + AppVersion + "</B>"));

			//Add the ImageButtons Cell
			_Row.Cells.Add(new TableCell());
			TableCell _HelpPrefCell = _Row.Cells[2];
			_HelpPrefCell.Attributes.Add("align", "right");
			_Row.Cells[2].Controls.Add(new LiteralControl("<a href=\"javascript:OpenWindow('/Vtk/Utility/UserPreference.aspx', 400, 300)\"><img src=\"/VTK/Images/Options.gif\" border=\"0\">"));
			_Row.Cells[2].Controls.Add(new LiteralControl("<a href=\"javascript:OpenWindow('/Vtk/Help/help.aspx?page=" + HelpPage + "&activepage=" + CurrentPage + "')\"><img src=\"/VTK/Images/Help.gif\" border=\"0\"></a>"));
			
			return _Table;
		}
	}
	/// <summary>
	/// Used by the <see cref="Vha.Vtk.WebControlLibrary.VtkPageHeader"/> control to display the control at design-time (in the web form
	/// designer). It shows enough of what the control will look like to make designing web forms
	/// easier.
	/// </summary>
	public class VtkPageHeaderDesigner : System.Web.UI.Design.ControlDesigner 
	{
		/// <summary>
		/// This method is called by VS.NET to display the control at design-time (in the web form
		/// designer). It shows enough of what the control will look like to make designing web forms
		/// easier.
		/// </summary>
		/// <returns>HTML string to display a facsimile of the control in the web page designer.</returns>
		public override string GetDesignTimeHtml()
		{
			return "<table width=\"100%\" bgcolor=\"#eeeeee\"><tr><td align=\"left\">Station: not selected</td><td align=\"right\"><img src=\"images/help.gif\" border=\"0\"/></td></tr></table>";
		}
	}

}